'''
Implementation of the WindowDiff segmentation evaluation metric described in
[PevznerHearst2002]_ with an optional modification to fix incorrect error
counting at the beginning and end of segmentations provided by 
[LamprierEtAl2007]_.

.. moduleauthor:: Chris Fournier <chris.m.fournier@gmail.com>
'''
#===============================================================================
# Copyright (c) 2011-2012, Chris Fournier
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the author nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#       
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#===============================================================================
from decimal import Decimal
from . import compute_window_size, parser_one_minus_support
from .. import SegmentationMetricError, compute_pairwise, \
    convert_masses_to_positions, compute_pairwise_values, create_tsv_rows
from ..data import load_file
from ..data.TSV import write_tsv
from ..data.Display import render_mean_values



def create_paired_window(hypothesis_positions, reference_positions, window_size,
                  lamprier_et_al_2007_fix):
    '''
    Create a set of pairs of units from each segmentation to go over using a
    window.
    '''
    phantom_size = window_size - 1
    phantom_size = 1 if phantom_size <= 0 else phantom_size
    if lamprier_et_al_2007_fix == False:
        units_ref_hyp = zip(reference_positions, hypothesis_positions)
    else:
        phantom = [0] * phantom_size
        units_ref_hyp = zip(phantom + reference_positions + phantom,
                            phantom + hypothesis_positions + phantom)
    return units_ref_hyp, phantom_size


def window_diff(hypothesis_positions, reference_positions, window_size=None,
                one_minus=False, lamprier_et_al_2007_fix=False,
                convert_from_masses=False):
    '''
    Calculates the WindowDiff segmentation evaluation metric score for a
    hypothetical segmentation against a reference segmentation for a given
    window size.  The standard method of calculating the window size
    is performed a window size is not specified.
    
    :param hypothesis_positions:     Hypothesis segmentation section labels
                                        sequence.
    :param reference_positions:      Reference segmentation section labels
                                        sequence.
    :param window_size:              The size of the window that is slid over \
                                        the two segmentations used to count \
                                        mismatches (default is None and will \
                                        use the average window size)
    :param one_minus:                Return 1-WindowDiff to make it no longer \
                                         a penalty-metric.
    :param lamprier_et_al_2007_fix:  Apply a fix for improperly counted errors \
                                        at the beginning and end of \
                                        segmentations, provided by \
                                        _[LamprierEtAl2007].
    :param convert_from_masses:      Convert the segmentations provided from \
                                        masses into positions.
    :type hypothesis_positions: list
    :type reference_positions: list
    :type window_size: int
    :type one_minus: bool
    :type lamprier_et_al_2007_fix: bool
    :type convert_from_masses: bool
    
    .. note:: See :func:`segeval.convert_masses_to_positions` for an example of
              the input format.
    '''
    # pylint: disable=C0103,R0913,R0914
    # Convert from masses into positions 
    if convert_from_masses:
        reference_positions  = convert_masses_to_positions(reference_positions)
        hypothesis_positions = convert_masses_to_positions(hypothesis_positions)
    # Check for input errors
    if len(reference_positions) != len(hypothesis_positions):
        raise SegmentationMetricError(
                    'Reference and hypothesis segmentations differ in length.')
    # Compute window size to use if unspecified
    if window_size is None:
        window_size = compute_window_size(reference_positions)
    # Create a set of pairs of units from each segmentation to go over using a
    # window
    units_ref_hyp, phantom_size = create_paired_window(hypothesis_positions,
                                                       reference_positions,
                                                       window_size,
                                                       lamprier_et_al_2007_fix)
    # Slide window over and sum the number of varying windows
    sum_differences = 0
    for i in xrange(0, len(units_ref_hyp) - window_size + 1):
        window = units_ref_hyp[i:i+window_size]
        ref_boundaries = 0
        hyp_boundaries = 0
        # For pair in window
        for j in xrange(0, len(window)-1):
            ref_part, hyp_part = zip(*window[j:j+2])
            # Boundary exists in the reference segmentation
            if ref_part[0] != ref_part[1]:
                ref_boundaries += 1
            # Boundary exists in the hypothesis segmentation
            if hyp_part[0] != hyp_part[1]:
                hyp_boundaries += 1
        # If the number of boundaries per segmentation in the window differs
        if ref_boundaries != hyp_boundaries:
            sum_differences += 1
    # Perform final division
    n = len(reference_positions) + 1
    if lamprier_et_al_2007_fix:
        n += (phantom_size * 2)
    win_diff = Decimal(sum_differences) / (n - window_size)
    if win_diff > 1:
        raise SegmentationMetricError('Incorrect value calculated.')
    if not one_minus:
        return win_diff
    else:
        return Decimal('1.0') - win_diff


def pairwise_window_diff(dataset_masses, one_minus=False,
                         lamprier_et_al_2007_fix=False,
                         window_size=None,
                         convert_from_masses=True):
    '''
    Calculate mean pairwise segmentation F-Measure.
    
    .. seealso:: :func:`window_diff`
    .. seealso:: :func:`segeval.compute_pairwise`
    
    :param dataset_masses: Segmentation mass dataset (including multiple \
                           codings).
    :type dataset_masses: dict
        
    :returns: Mean, standard deviation, variance, and standard error of a \
        segmentation metric.
    :rtype: :class:`decimal.Decimal`, :class:`decimal.Decimal`, \
        :class:`decimal.Decimal`, :class:`decimal.Decimal`
    '''
    def wrapper(hypothesis_masses, reference_masses):
        '''
        Wrapper to provide parameters.
        '''
        return window_diff(hypothesis_masses, reference_masses,
                           window_size=window_size, one_minus=one_minus,
                           lamprier_et_al_2007_fix=lamprier_et_al_2007_fix,
                           convert_from_masses=convert_from_masses)
    
    return compute_pairwise(dataset_masses, wrapper, permuted=True)


OUTPUT_NAME = 'Mean WindowDiff'
SHORT_NAME  = 'WindowDiff'


def values_window_diff(dataset_masses, name, one_minus,
                       lamprier_et_al_2007_fix):
    '''
    Produces a TSV for this metric
    '''
    # Define a fnc to pass parameters
    def wrapper(hypothesis_masses, reference_masses):
        '''
        Wrapper to provide parameters.
        '''
        return window_diff(hypothesis_masses, reference_masses,
                           one_minus=one_minus, convert_from_masses=True,
                           lamprier_et_al_2007_fix=lamprier_et_al_2007_fix)
    # Get values
    header = list(['coder1', 'coder2', name])
    values = compute_pairwise_values(dataset_masses, wrapper, permuted=True)
    return create_tsv_rows(header, values)


def parse(args):
    '''
    Parse this module's metric arguments and perform requested actions.
    '''
    output = None
    values = load_file(args)[0]
    one_minus = args['oneminus']
    lamprier_et_al_2007_fix = args['lamprier_et_al_2007']
    name = SHORT_NAME
    
    if one_minus:
        name = '1 - %s' % name
    
    # Is a TSV requested?
    if args['output'] != None:
        # Create a TSV
        output_file = args['output'][0]
        header, rows = values_window_diff(values, name, one_minus,
                                          lamprier_et_al_2007_fix)
        write_tsv(output_file, header, rows)
    else:
        # Create a string to output
        mean, std, var, stderr = pairwise_window_diff(values, one_minus)
        output = render_mean_values(name, mean, std, var, stderr)
    return output


def create_parser(subparsers):
    '''
    Setup a command line parser for this module's metric.
    '''
    from ..data import parser_add_file_support
    parser = subparsers.add_parser('wd',
                                   help=OUTPUT_NAME)
    parser_add_file_support(parser)
    parser_one_minus_support(parser)
    
    parser.add_argument('--lamprier_et_al_2007',
                        default=False,
                        action='store_true',
                        help='Applies the phantom-boundary fix from Lamprier \
                            et al. (2007) to properly count errors at the \
                            beginning and end of a segmentation; default is \
                            false to remain canon with existing reported \
                            values.')
    
    parser.set_defaults(func=parse)




'''
Tests the WindowDiff evaluation metric.

.. moduleauthor:: Chris Fournier <chris.m.fournier@gmail.com>
'''
#===============================================================================
# Copyright (c) 2011-2012, Chris Fournier
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the author nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#       
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#===============================================================================
import unittest
from decimal import Decimal
from .FbMeasure import f_b_measure, pairwise_f_b_measure
from ..data.Samples import KAZANTSEVA2012_G5, KAZANTSEVA2012_G2, \
    COMPLETE_AGREEMENT, LARGE_DISAGREEMENT
from .. import convert_positions_to_masses


class TestFbMeasure(unittest.TestCase):
    '''
    Test segmentation F-measure.
    '''
    # pylint: disable=R0904

    def test_identical(self):
        '''
        Test whether identical segmentations produce 1.0.
        '''
        # pylint: disable=C0324,C0103
        a = convert_positions_to_masses([1,1,1,1,1,2,2,2,3,3,3,3,3])
        b = convert_positions_to_masses([1,1,1,1,1,2,2,2,3,3,3,3,3])
        self.assertEqual(f_b_measure(a, b), 1.0)

    def test_no_boundaries(self):
        '''
        Test whether no segments versus some segments produce 0.0.
        '''
        # pylint: disable=C0324,C0103
        a = convert_positions_to_masses([1,1,1,1,1,1,1,1,1,1,1,1,1])
        b = convert_positions_to_masses([1,1,1,1,2,2,2,2,3,3,3,3,3])
        self.assertEqual(f_b_measure(a,b), 0)
        self.assertEqual(f_b_measure(b,a), 0)

    def test_all_boundaries(self):
        '''
        Test whether all segments versus some segments produces 2/12, or 0.167.
        '''
        # pylint: disable=C0324,C0103
        a = convert_positions_to_masses([1,2,3,4,5,6,7,8,9,10,11,12,13])
        b = convert_positions_to_masses([1,1,1,1,2,2,2,2,3,3,3,3,3])
        self.assertEqual(f_b_measure(a,b),
                         Decimal('0.2857142857142857142857142857'))
        self.assertEqual(f_b_measure(b,a),
                         Decimal('0.2857142857142857142857142857'))

    def test_all_and_no_boundaries(self):
        '''
        Test whether all segments versus no segments produces 0.0.
        '''
        # pylint: disable=C0324,C0103
        a = convert_positions_to_masses([1,2,3,4,5,6,7,8,9,10,11,12,13])
        b = convert_positions_to_masses([1,1,1,1,1,1,1,1,1,1,1,1,1])
        self.assertEqual(f_b_measure(a,b), 0)
        self.assertEqual(f_b_measure(b,a), 0)

    def test_translated_boundary(self):
        '''
        Test whether 2/3 total segments participate in mis-alignment produces
        0.5.
        '''
        # pylint: disable=C0324,C0103
        a = convert_positions_to_masses([1,1,1,1,1,2,2,2,3,3,3,3,3])
        b = convert_positions_to_masses([1,1,1,1,2,2,2,2,3,3,3,3,3])
        self.assertEqual(f_b_measure(a,b),
                         Decimal('0.5'))
        self.assertEqual(f_b_measure(b,a),
                         Decimal('0.5'))
    
    def test_extra_boundary(self):
        '''
        Test whether 1/3 segments that are non-existent produces 0.8.
        '''
        # pylint: disable=C0324,C0103
        a = convert_positions_to_masses([1,1,1,1,1,2,2,2,3,3,3,3,3])
        b = convert_positions_to_masses([1,1,1,1,1,2,3,3,4,4,4,4,4])
        self.assertEqual(f_b_measure(a,b),
                         Decimal('0.8'))
        self.assertEqual(f_b_measure(b,a),
                         Decimal('0.8'))
    
    def test_full_miss_and_misaligned(self):
        '''
        Test whether a full miss and a translated boundary out of 4 produces
        0.25. 
        '''
        # pylint: disable=C0324,C0103
        a = convert_positions_to_masses([1,1,1,1,2,2,2,2,3,3,3,3,3])
        b = convert_positions_to_masses([1,1,1,1,1,2,3,3,4,4,4,4,4])
        self.assertEqual(f_b_measure(a,b), Decimal('0.4'))
        self.assertEqual(f_b_measure(b,a), Decimal('0.4'))


class TestPairwiseFbMeasure(unittest.TestCase):
    # pylint: disable=R0904
    '''
    Test pairwise F_b_measure.
    '''

    def test_kazantseva2012_g5(self):
        '''
        Calculate permuted pairwise percentage on Group 5 from the dataset
        collected in [KazantsevaSzpakowicz2012]_.
        '''
        self.assertEqual(pairwise_f_b_measure(KAZANTSEVA2012_G5),
                         (Decimal('0.2441586812212541867438777317'),
                          Decimal('0.2305169230001435997031211478'),
                          Decimal('0.05313805178945413332341442160'),
                          Decimal('0.03327225188672428552085603181')))
    
    def test_kazantseva2012_g2(self):
        '''
        Calculate mean permuted pairwise percentage on Group 2 from the dataset
        collected in [KazantsevaSzpakowicz2012]_.
        '''
        self.assertEqual(pairwise_f_b_measure(KAZANTSEVA2012_G2),
                         (Decimal('0.4777419851104061630377419851'),
                          Decimal('0.2090943663384804017117580489'),
                          Decimal('0.04372045403449064611325971031'),
                          Decimal('0.01908761684847243358481821492')))
    
    def test_large_disagreement(self):
        '''
        Calculate mean permuted pairwise percentage on a theoretical dataset
        containing large disagreement.
        '''
        self.assertEqual(pairwise_f_b_measure(LARGE_DISAGREEMENT),
                         (0.0,
                          0.0,
                          0.0,
                          0.0))
    
    def test_complete_agreement(self):
        '''
        Calculate mean permuted pairwise percentage on a theoretical dataset
        containing complete agreement.
        '''
        self.assertEqual(pairwise_f_b_measure(COMPLETE_AGREEMENT),
                         (1.0,
                          0.0,
                          0.0,
                          0.0))




'''
Segmentation evaluation metric package. Provides evaluation metrics to
evaluate the performance of both human and automatic text (i.e., discourse)
segmenters.  This package contains a new metric called Segmentation Similarity
(S) [FournierInkpen2012]_ which is recommended for usage along with a variety
of inter-coder agreement coefficients that utilize S.

To use S, see the :mod:`segeval.similarity` module.

.. moduleauthor:: Chris Fournier <chris.m.fournier@gmail.com>
'''
#===============================================================================
# Copyright (c) 2011-2012, Chris Fournier
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the author nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#       
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#===============================================================================
import csv
import os


def write_tsv(filepath, header, rows):
    '''
    Write a TSV file using the given header and rows.
    
    :param filepath: Path and filename of a file to write to
    :param header:   List of category names
    :param rows:     Data to write for all categories
    :type rows:   str
    :type header: :class:`list`
    :type rows:   :class:`list` of :class:`list`
    '''
    # Create a default filename if a dir is specified
    if os.path.isdir(filepath):
        filepath = os.path.join(filepath, 'output.tsv')
    # Open file
    tsv = csv.writer(open(filepath, 'wb'), delimiter='\t', quotechar='"',
                     quoting=csv.QUOTE_MINIMAL)
    tsv.writerow(header)
    for row in rows:
        tsv.writerow(row)




'''
Tests the similarity evaluation metric functions.

.. moduleauthor:: Chris Fournier <chris.m.fournier@gmail.com>
'''
#===============================================================================
# Copyright (c) 2011-2012, Chris Fournier
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the author nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#       
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#===============================================================================
import unittest
from decimal import Decimal
from .SegmentationSimilarity import similarity
from .. import SegmentationMetricError


class TestSegmentationSimilarity(unittest.TestCase):
    '''
    Tests the Segmentation Similarity metric.
    '''
    # pylint: disable=R0904,C0324
    
    N                    = 2
    WEIGHT               = (1,1)
    SCALE_TRANSPOSITIONS = True
    
    REF_A = [1,2,2,4,2,1]
    REF_B = [1,2,3,3,2,1]
    REF_C = [2,1,3,3,2,1]
    REF_D = [2,1,6,2,1]
    REF_E = [3,6,2,1]
    REF_F = [2,2,5,2,1]
    
    NO_REF_SEG  = [12]
    ALL_REF_SEG = [1,1,1,1,1,1,1,1,1,1,1,1]
    
    TOTAL_REFS_MASS = 12
    ALL_REFS = [REF_A,REF_B,REF_C,REF_D,REF_E,REF_F,NO_REF_SEG,ALL_REF_SEG]
    
    REF_TRANS_A = [2,2,2]
    REF_TRANS_B = [1,2,3]
    REF_TRANS_C = [3,2,1]
    REF_TRANS_D = [2,2,2]
    
    SKIP = False
    
    
    def test_verify_total_mass(self):
        '''
        Tests to ensure that all of the cases of the same hypothetical problem
        sum to the same mass.
        '''
        if TestSegmentationSimilarity.SKIP:
            return
        # Check for TOTAL_MASS
        for ref in TestSegmentationSimilarity.ALL_REFS:
            self.assertTrue(TestSegmentationSimilarity.TOTAL_REFS_MASS,
                            sum(ref))
    
    
    def test_empty_set(self):
        '''
        In this case the method is expected to produce a similarity of 1.0.
        '''
        # pylint: disable=C0103
        if TestSegmentationSimilarity.SKIP:
            return
        # Two empty sets
        sim = similarity([], [],
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, 1.0)
        u,t = similarity([], [],
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS,
                         return_parts=True)
        self.assertEqual((u,t), (0,0))
        
    
    def test_exception_on_mismatch(self):
        '''
        Test that mismatched mass totals raise an exception.
        '''
        if TestSegmentationSimilarity.SKIP:
            return
        self.assertRaises(SegmentationMetricError, similarity, [1], [])
        self.assertRaises(SegmentationMetricError, similarity, [], [1])
        self.assertRaises(SegmentationMetricError, similarity,
                          TestSegmentationSimilarity.REF_A,
                          [1] + TestSegmentationSimilarity.REF_A)
    
    
    def test_exception_on_zero(self):
        '''
        Test that otherwise conforming masses that contain zeros raise an
        exception.
        '''
        if TestSegmentationSimilarity.SKIP:
            return
        self.assertRaises(SegmentationMetricError, similarity, [0], [0])
        self.assertRaises(SegmentationMetricError, similarity,
                          [1,2,3],
                          [1,0,5])
        self.assertRaises(SegmentationMetricError, similarity,
                          [1,0,5],
                          [1,2,3])
    
    
    def test_reflexivity_and_full_match(self):
        '''
        Test the reflexive property, and the value.
        '''
        if TestSegmentationSimilarity.SKIP:
            return
        for ref in TestSegmentationSimilarity.ALL_REFS:
            self.assertEqual(similarity(ref,ref), similarity(ref,ref))
            self.assertEqual(similarity(ref,ref), 1.0)
        sim = similarity(TestSegmentationSimilarity.NO_REF_SEG,
                         TestSegmentationSimilarity.NO_REF_SEG,
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, 1.0)
        sim = similarity(TestSegmentationSimilarity.ALL_REF_SEG,
                         TestSegmentationSimilarity.ALL_REF_SEG,
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, 1.0)
    
    
    def test_symmetry_property(self):
        '''
        Test the symmetry property.
        '''
        if TestSegmentationSimilarity.SKIP:
            return
        refs = TestSegmentationSimilarity.ALL_REFS
        for i,ref in enumerate(refs):
            not_refs = [ref for j, ref in enumerate(refs) if j != i]
            for not_ref in not_refs:
                self.assertEqual(
                    similarity(ref,not_ref,
                               TestSegmentationSimilarity.N,
                               TestSegmentationSimilarity.WEIGHT,
                               TestSegmentationSimilarity.SCALE_TRANSPOSITIONS),
                    similarity(not_ref,ref,
                               TestSegmentationSimilarity.N,
                               TestSegmentationSimilarity.WEIGHT,
                               TestSegmentationSimilarity.SCALE_TRANSPOSITIONS))
    
    
    def test_with_no_segmentation(self):
        '''
        Test cases compared to a single segment (i.e. no segmentation).
        '''
        if TestSegmentationSimilarity.SKIP:
            return
        sim = similarity(TestSegmentationSimilarity.REF_A,
                         TestSegmentationSimilarity.NO_REF_SEG,
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, Decimal('0.5454545454545454545454545455'))
        sim = similarity(TestSegmentationSimilarity.REF_B,
                         TestSegmentationSimilarity.NO_REF_SEG,
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, Decimal('0.5454545454545454545454545455'))
        sim = similarity(TestSegmentationSimilarity.REF_C,
                         TestSegmentationSimilarity.NO_REF_SEG,
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, Decimal('0.5454545454545454545454545455'))
        sim = similarity(TestSegmentationSimilarity.REF_E,
                         TestSegmentationSimilarity.NO_REF_SEG,
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, Decimal('0.7272727272727272727272727273'))
    
     
    def test_with_maximal_segmentation(self):
        '''
        Test cases compared to a fully segmented case (each segment is one unit
        long).
        '''
        if TestSegmentationSimilarity.SKIP:
            return
        sim = similarity(TestSegmentationSimilarity.REF_A,
                         TestSegmentationSimilarity.ALL_REF_SEG,
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, Decimal('0.4545454545454545454545454545'))
        sim = similarity(TestSegmentationSimilarity.REF_D,
                         TestSegmentationSimilarity.ALL_REF_SEG,
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, Decimal('0.3636363636363636363636363636'))
        sim = similarity(TestSegmentationSimilarity.REF_E,
                         TestSegmentationSimilarity.ALL_REF_SEG,
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, Decimal('0.2727272727272727272727272727'))
    
    
    def test_maximal_and_no_segments(self):
        '''
        Test a case of one segment against a completely segmented case).
        '''
        if TestSegmentationSimilarity.SKIP:
            return
        sim = similarity(TestSegmentationSimilarity.NO_REF_SEG,
                         TestSegmentationSimilarity.ALL_REF_SEG,
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, 0.0)
        sim = similarity([10],
                         [1,1,1,1,1,1,1,1,1,1],
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, 0.0)
    
    
    def test_simple_translation(self):
        '''
        Test a case where segment boundaries are shifted once in one direction.
        '''
        if TestSegmentationSimilarity.SKIP:
            return
        sim_ab = similarity(TestSegmentationSimilarity.REF_TRANS_A,
                            TestSegmentationSimilarity.REF_TRANS_B,
                            TestSegmentationSimilarity.N,
                            TestSegmentationSimilarity.WEIGHT,
                            TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        sim_cd = similarity(TestSegmentationSimilarity.REF_TRANS_C,
                            TestSegmentationSimilarity.REF_TRANS_D,
                            TestSegmentationSimilarity.N,
                            TestSegmentationSimilarity.WEIGHT,
                            TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim_ab, sim_cd)
        self.assertEqual(sim_ab, Decimal('0.6'))
    
    
    def test_erratic_translation(self):
        '''
        Test a case where segment boundaries are shifted erratically.
        '''
        if TestSegmentationSimilarity.SKIP:
            return
        # AB
        sim = similarity(TestSegmentationSimilarity.REF_A,
                         TestSegmentationSimilarity.REF_B,
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, Decimal('0.9090909090909090909090909091'))
        # AC
        sim = similarity(TestSegmentationSimilarity.REF_A,
                         TestSegmentationSimilarity.REF_C,
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, Decimal('0.8181818181818181818181818182'))
        # AD
        sim = similarity(TestSegmentationSimilarity.REF_A,
                         TestSegmentationSimilarity.REF_D,
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, Decimal('0.8181818181818181818181818182'))
    
    
    def test_differing_num_of_segments(self):
        '''
        Test a case where the number of segments differs.
        '''
        if TestSegmentationSimilarity.SKIP:
            return
        # AE
        sim = similarity(TestSegmentationSimilarity.REF_A,
                         TestSegmentationSimilarity.REF_E,
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, Decimal('0.8181818181818181818181818182'))
        # AF
        sim = similarity(TestSegmentationSimilarity.REF_A,
                         TestSegmentationSimilarity.REF_F,
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, Decimal('0.7272727272727272727272727273'))
        # Whiteboard example
        sim = similarity([2,1,2,2], [4,2,1],
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, Decimal('0.5'))


class TestSegmentationSimilarityFigures(unittest.TestCase):
    '''
    Tests the Segmentation Similarity metric figures.
    '''
    # pylint: disable=R0904,C0324
    
    def test_figure_walkthrough(self):
        '''
        Tests the case in the Figure in the paper's walkthrough.
        '''
        if TestSegmentationSimilarity.SKIP:
            return
        sim = similarity([1,2,2,3,3,1,2],
                         [1,2,1,2,6,2],
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, Decimal('0.6923076923076923076923076923'))
    
    
    def test_figure_insert_delete(self):
        '''
        Tests the case in the paper's insertion/deletion Figure.
        '''
        if TestSegmentationSimilarity.SKIP:
            return
        sim = similarity([1,2,2,4,2,1],
                         [1,2,6,2,1],
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, Decimal('0.9090909090909090909090909091'))
    
    
    def test_figure_complex_ins_del(self):
        '''
        Tests the case in the paper's complex insertion/deletion Figure.
        '''
        if TestSegmentationSimilarity.SKIP:
            return
        sim = similarity([1,1,3,1,2,2,1,1],
                         [2,1,4,2,1,2],
                         TestSegmentationSimilarity.N,
                         TestSegmentationSimilarity.WEIGHT,
                         TestSegmentationSimilarity.SCALE_TRANSPOSITIONS)
        self.assertEqual(sim, Decimal('0.4545454545454545454545454545'))
        
    
    def test_scaled_transposition(self):
        '''
        Tests the case in the Figure in the paper's walkthrough.
        '''
        #if TestSegmentationSimilarity.SKIP:
        #    return
        sim = similarity([1,4,4],
                         [1,2,6],
                         5,
                         TestSegmentationSimilarity.WEIGHT,
                         True)
        self.assertEqual(sim, Decimal('0.8125'))
        
    
    def test_figure_walkthrough_and_F(self):
        '''
        Tests the case in the Figure in the paper's walkthrough.
        '''
        # pylint: disable=C0103
        if TestSegmentationSimilarity.SKIP:
            return
        a   = [1,2,2,3,3,1,2]
        f_a = [sum(a)]#[1] * sum(a)
        b   = [1,2,1,2,6,2]
        f_b = [sum(a)]#[1] * sum(a)
        sim_ab_u = \
            similarity(a,
                       b,
                       TestSegmentationSimilarity.N,
                       TestSegmentationSimilarity.WEIGHT,
                       TestSegmentationSimilarity.SCALE_TRANSPOSITIONS,
                       return_parts=True)[0]
        sim_afb_u = \
            similarity(a,
                       f_b,
                       TestSegmentationSimilarity.N,
                       TestSegmentationSimilarity.WEIGHT,
                       TestSegmentationSimilarity.SCALE_TRANSPOSITIONS,
                       return_parts=True)[0]
        sim_fab_u = \
            similarity(f_a,
                       b,
                       TestSegmentationSimilarity.N,
                       TestSegmentationSimilarity.WEIGHT,
                       TestSegmentationSimilarity.SCALE_TRANSPOSITIONS,
                       return_parts=True)[0]
        self.assertEqual(sim_ab_u,  Decimal('9'))
        self.assertEqual(sim_afb_u, Decimal('7'))
        self.assertEqual(sim_fab_u, Decimal('8'))
        



'''
Tests for the console commands provided by __main__.py

.. moduleauthor:: Chris Fournier <chris.m.fournier@gmail.com>
'''
#===============================================================================
# Copyright (c) 2012, Chris Fournier
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the author nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#       
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#===============================================================================
import os
import unittest
from .__main__ import main


class TestMain(unittest.TestCase):
    '''
    Test command line functions.
    '''
    # pylint: disable=R0904,C0301
    
    test_data_dir = os.path.join(os.path.split(__file__)[0], 'data')
    
    print_output = True
    test_help    = False
    
    EXPECTED_FILE_STRINGS = [
'Pi*_s = 0.7165532879818594104308390023',
'K*_s = 0.7170345217883418222976796831',
'B_s = 0.0014285714285714285714285714',
'F_1 \n \tmean\t= 0.5488257630783062008183635783\n\tstd\t= 0.1541776268839723153329934675\n\tvar\t= 0.02377074063157338283693571885\n\tstderr\t= 0.02379012431740241024748814014',
'P \n \tmean\t= 0.5705026455026455026455026457\n\tstd\t= 0.1913327668048554567150062580\n\tvar\t= 0.03660822765320119817474314830\n\tstderr\t= 0.04175223270517067477184376667',
'R \n \tmean\t= 0.5708994708994708994708994710\n\tstd\t= 0.2009486961610950520518783548\n\tvar\t= 0.0403803784888440973097057753\n\tstderr\t= 0.04385060052194762366757538877',
'Pr \n \tmean\t= 0.3933140933140933140933140934\n\tstd\t= 0.1432599301657948205815072907\n\tvar\t= 0.02052340759110840880922651004\n\tstderr\t= 0.03126187971613534798006584564',
'S \n \tmean\t= 0.7619047619047619047619047619\n\tstd\t= 0.07055015423823358837798727192\n\tvar\t= 0.004977324263038548752834467119\n\tstderr\t= 0.01539530581369118988034410932',
'Pk \n \tmean\t= 0.3258145363408521303258145360\n\tstd\t= 0.08839695282480399887931269980\n\tvar\t= 0.007814021268710623676986953602\n\tstderr\t= 0.01363994594730826050014572596',
'WindowDiff \n \tmean\t= 0.3047619047619047619047619048\n\tstd\t= 0.08438116736509214477138630998\n\tvar\t= 0.007120181405895691609977324262\n\tstderr\t= 0.01302029679814566213216037145']

    EXPECTED_FOLDER_STRINGS = [
'Mean Pi*_s \n \tmean\t= 0.7165532879818594104308390023\n\tstd\t= 0\n\tvar\t= 0\n\tstderr\t= 0',
'Mean K*_s \n \tmean\t= 0.7170345217883418222976796831\n\tstd\t= 0\n\tvar\t= 0\n\tstderr\t= 0',
'Mean B_s \n \tmean\t= 0.0014285714285714285714285714\n\tstd\t= 0\n\tvar\t= 0\n\tstderr\t= 0',
'F_1 \n \tmean\t= 0.5488257630783062008183635783\n\tstd\t= 0.1541776268839723153329934675\n\tvar\t= 0.02377074063157338283693571885\n\tstderr\t= 0.02379012431740241024748814014',
'P \n \tmean\t= 0.5705026455026455026455026457\n\tstd\t= 0.1913327668048554567150062580\n\tvar\t= 0.03660822765320119817474314830\n\tstderr\t= 0.04175223270517067477184376667',
'R \n \tmean\t= 0.5708994708994708994708994710\n\tstd\t= 0.2009486961610950520518783548\n\tvar\t= 0.04038037848884409730970577530\n\tstderr\t= 0.04385060052194762366757538877',
'Pr \n \tmean\t= 0.3933140933140933140933140934\n\tstd\t= 0.1432599301657948205815072907\n\tvar\t= 0.02052340759110840880922651004\n\tstderr\t= 0.03126187971613534798006584564',
'S \n \tmean\t= 0.7619047619047619047619047619\n\tstd\t= 0.07055015423823358837798727192\n\tvar\t= 0.004977324263038548752834467119\n\tstderr\t= 0.01539530581369118988034410932',
'Pk \n \tmean\t= 0.3258145363408521303258145360\n\tstd\t= 0.08839695282480399887931269982\n\tvar\t= 0.007814021268710623676986953605\n\tstderr\t= 0.01363994594730826050014572597',
'WindowDiff \n \tmean\t= 0.3047619047619047619047619048\n\tstd\t= 0.08438116736509214477138630999\n\tvar\t= 0.007120181405895691609977324264\n\tstderr\t= 0.01302029679814566213216037145']

    EXPECTED_OM_STRINGS = [
'1 - Pk \n \tmean\t= 0.6741854636591478696741854645\n\tstd\t= 0.08839695282480399887931269980\n\tvar\t= 0.007814021268710623676986953602\n\tstderr\t= 0.01363994594730826050014572596',
'1 - WindowDiff \n \tmean\t= 0.6952380952380952380952380952\n\tstd\t= 0.08438116736509214477138630998\n\tvar\t= 0.007120181405895691609977324262\n\tstderr\t= 0.01302029679814566213216037145']
    
    EXPECTED_WPR_STRINGS = [
'WinPR-f_1 \n \tmean\t= 0.6537078835417392808312562948\n\tstd\t= 0.1104334172259618258006709431\n\tvar\t= 0.01219553964020336212051245300\n\tstderr\t= 0.02409854731860048794927717470',
'WinPR-p \n \tmean\t= 0.6700319521748093176664605238\n\tstd\t= 0.1469987063321598074606894023\n\tvar\t= 0.0216086196633285598741911137\n\tstderr\t= 0.03207774756322412707724005526',
'WinPR-r \n \tmean\t= 0.6683725005153576582148010719\n\tstd\t= 0.1570297604076544084333187669\n\tvar\t= 0.02465834565368534800463416065\n\tstderr\t= 0.03426670302042171194576414586']

    def test_load_files(self):
        '''
        Test the different ways to load files.
        '''
        metric = 'pi'
        argv = [metric, os.path.join(self.test_data_dir,
                                     'hearst1997.json')]
        self.assertEqual('Pi*_s = 0.7165532879818594104308390023', main(argv))
        
        argv = [metric, '-f', 'json', os.path.join(self.test_data_dir,
                                                   'hearst1997.json')]
        self.assertEqual('Pi*_s = 0.7165532879818594104308390023', main(argv))
        
        argv = [metric, '-f', 'tsv', os.path.join(self.test_data_dir,
                                                  'hearst1997.tsv')]
        self.assertEqual('Pi*_s = 0.7165532879818594104308390023', main(argv))


    def test_all_but_winpr_folder(self):
        '''
        Run through each metric and load from a file.
        '''
        metrics = ['pi', 'k', 'b', 'f', 'p', 'r', 'pr', 's', 'pk', 'wd']
        for metric, expected in zip(metrics, self.EXPECTED_FOLDER_STRINGS):
            argv = [metric, os.path.join(self.test_data_dir,
                                         '..')]
            actual = main(argv)
            self.assertEqual(expected, actual)


    def test_all_but_winpr_metrics(self):
        '''
        Run through each metric and load from a file.
        '''
        metrics = ['pi', 'k', 'b', 'f', 'p', 'r', 'pr', 's', 'pk', 'wd']
        for metric, expected in zip(metrics, self.EXPECTED_FILE_STRINGS):
            argv = [metric, os.path.join(self.test_data_dir,
                                         'hearst1997.json')]
            actual = main(argv)
            self.assertEqual(expected, actual)


    def test_winpr_metrics(self):
        '''
        Run through each metric and load from a file.
        '''
        submetrics = ['f', 'p', 'r']
        for submetric, expected in zip(submetrics, self.EXPECTED_WPR_STRINGS):
            argv = ['wpr', submetric, os.path.join(self.test_data_dir,
                                                   'hearst1997.json')]
            actual = main(argv)
            self.assertEqual(expected, actual)


    def test_om_metrics(self):
        '''
        Run through each metric and load from a file.
        '''
        metrics = ['pk', 'wd']
        for metric, expected in zip(metrics, self.EXPECTED_OM_STRINGS):
            argv = [metric, '-om', os.path.join(self.test_data_dir,
                                         'hearst1997.json')]
            actual = main(argv)
            self.assertEqual(expected, actual)


    def test_file(self):
        '''
        Run through each metric and load from a file.
        '''
        metrics = ['pi', 'k', 'b', 'f', 'p', 'r', 'pr', 's', 'pk', 'wd']
        filesizes = [51, 50, 49, 1524, 1524, 1524, 702, 584, 1747, 645]
        for metric, expected_filesize in zip(metrics, filesizes):
            filename = 'testfile.tsv'
            if os.path.exists(filename):
                os.remove(filename)
            self.assertFalse(os.path.exists(filename))
            argv = [metric, '-o', filename, os.path.join(self.test_data_dir,
                                                         'hearst1997.json')]
            try:
                main(argv)
                self.assertTrue(os.path.exists(filename))
                actual_filesize = len(open(filename).read())
                self.assertEqual(expected_filesize, actual_filesize, 
                                 '%(metric)s %(expected)i != %(actual)i' % \
                                 {'metric'   : metric,
                                  'expected' : expected_filesize,
                                  'actual'   : actual_filesize})
            finally:
                if os.path.exists(filename):
                    os.remove(filename)
            
            self.assertFalse(os.path.exists(filename))


    def test_file_om(self):
        '''
        Run through each metric and load from a file.
        '''
        metrics = ['pk', 'wd']
        filesizes = [1751, 649]
        for metric, expected_filesize in zip(metrics, filesizes):
            filename = 'testfile.tsv'
            if os.path.exists(filename):
                os.remove(filename)
            self.assertFalse(os.path.exists(filename))
            argv = [metric, '-om', '-o', filename,
                    os.path.join(self.test_data_dir, 'hearst1997.json')]
            try:
                main(argv)
                self.assertTrue(os.path.exists(filename))
                actual_filesize = len(open(filename).read())
                self.assertEqual(expected_filesize, actual_filesize, 
                                 '%(metric)s %(expected)i != %(actual)i' % \
                                 {'metric'   : metric,
                                  'expected' : expected_filesize,
                                  'actual'   : actual_filesize})
            finally:
                if os.path.exists(filename):
                    os.remove(filename)
            
            self.assertFalse(os.path.exists(filename))


    def test_file_winpr(self):
        '''
        Run through each metric and load from a file.
        '''
        submetrics = ['f', 'p', 'r']
        filesizes = [1741, 1741, 1741]
        for submetric, expected_filesize in zip(submetrics, filesizes):
            
            filename = 'testfile.tsv'
            if os.path.exists(filename):
                os.remove(filename)
            self.assertFalse(os.path.exists(filename))
            argv = ['wpr', submetric, '-o', filename,
                    os.path.join(self.test_data_dir, 'hearst1997.json')]
            try:
                main(argv)
                self.assertTrue(os.path.exists(filename))
                actual_filesize = len(open(filename).read())
                self.assertEqual(expected_filesize, actual_filesize, 
                                 '%(metric)s %(expected)i != %(actual)i' % \
                                 {'metric'   : submetric,
                                  'expected' : expected_filesize,
                                  'actual'   : actual_filesize})
            finally:
                if os.path.exists(filename):
                    os.remove(filename)
            
            self.assertFalse(os.path.exists(filename))


    def test_file_s_detailed(self):
        '''
        Test detailed S output.
        '''
        expected_filesize = 2131
        filename = 'testfile.tsv'
        if os.path.exists(filename):
            os.remove(filename)
        self.assertFalse(os.path.exists(filename))
        argv = ['s', '-de', '-o', filename, os.path.join(self.test_data_dir,
                                                         'hearst1997.json')]
        try:
            main(argv)
            self.assertTrue(os.path.exists(filename))
            actual_filesize = len(open(filename).read())
            self.assertEqual(expected_filesize, actual_filesize, 
                             '%(metric)s %(expected)i != %(actual)i' % \
                             {'metric'   : 's',
                              'expected' : expected_filesize,
                              'actual'   : actual_filesize})
        finally:
            if os.path.exists(filename):
                os.remove(filename)
        
        self.assertFalse(os.path.exists(filename))

    def test_help_output(self):
        '''
        Test the help output.
        '''
        argv = ['wd', '-h']
        if self.print_output and self.test_help:
            main(argv)




'''
Utility functions for the package.

.. codeauthor:: Chris Fournier <chris.m.fournier@gmail.com>
'''
#===============================================================================
# Copyright (c) 2012, Chris Fournier
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the author nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#       
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#===============================================================================
import os


ROOT_PACKAGE_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                '../'))


def default_load_tests(cur_file, loader, tests):
    '''
    Default functionality for module load_tests functions which 
    are contained in each module's __init__.py file)
    
    :param cur_file: __file__ from the calling module.
    :param loader: Test loader.
    :param tests: Test suite.
    :type cur_file: str
    :type loader: str
    :type tests: unittest.TestSuite
    
    :returns: A modified test suite.
    :rtype: :class:`unittest.TestSuite`
    
    .. seealso:: The `load_tests protocol <http://docs.python.org/library/\
    unittest.html#load-tests-protocol>`_.
    '''
    pattern = '*Test.py'
    cur_dir = os.path.split(cur_file)[0]
    discovered_tests = loader.discover(cur_dir,
                                       pattern=pattern,
                                       top_level_dir=ROOT_PACKAGE_DIR)
    tests.addTests(discovered_tests)
    return tests




'''
Tests the WinPR evaluation metric.

.. moduleauthor:: Chris Fournier <chris.m.fournier@gmail.com>
'''
#===============================================================================
# Copyright (c) 2011-2012, Chris Fournier
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the author nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#       
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#===============================================================================
import unittest
from decimal import Decimal
from .WinPR import pairwise_win_pr, win_pr, win_pr_f
from ..data.Samples import KAZANTSEVA2012_G5, KAZANTSEVA2012_G2, \
    COMPLETE_AGREEMENT, LARGE_DISAGREEMENT


class TestWinPR(unittest.TestCase):
    '''
    Test WinPR.
    '''
    # pylint: disable=R0904,C0324

    def test_identical(self):
        '''
        Test whether identical segmentations produce 1.0.
        '''
        segs_a = [1,1,1,1,1,2,2,2,3,3,3,3,3]
        segs_b = [1,1,1,1,1,2,2,2,3,3,3,3,3]
        self.assertEqual(win_pr(segs_a, segs_b),
                         (4, 0, 0, 22) )
        self.assertEqual(win_pr_f(segs_a, segs_b),
                         Decimal('1') )

    def test_no_boundaries(self):
        '''
        Test whether no segments versus some segments produce 0.6667.
        '''
        segs_a = [1,1,1,1,2,2,2,2,3,3,3,3,3]
        segs_b = [1,1,1,1,1,1,1,1,1,1,1,1,1]
        self.assertEqual(win_pr(segs_a, segs_b),
                         (12, 12, 0, 67) )
        self.assertEqual(win_pr_f(segs_a, segs_b),
                         Decimal('0.6666666666666666666666666667') )
        self.assertEqual(win_pr(segs_b, segs_a),
                         (2, 0, 2, 22) )
        self.assertEqual(win_pr_f(segs_b, segs_a),
                         Decimal('0.6666666666666666666666666667') )

    def test_all_boundaries(self):
        '''
        Test whether all segments versus some segments produces 0.4444.
        '''
        segs_a = [1,1,1,1,2,2,2,2,3,3,3,3,3]
        segs_b = [1,2,3,4,5,6,7,8,9,10,11,12,13]
        self.assertEqual(win_pr(segs_a, segs_b),
                         (4, 0, 10, 12) )
        self.assertEqual(win_pr_f(segs_a, segs_b),
                         Decimal('0.4444444444444444444444444444') )
        self.assertEqual(win_pr(segs_b, segs_a),
                         (4, 10, 0, 12) )
        self.assertEqual(win_pr_f(segs_b, segs_a),
                         Decimal('0.4444444444444444444444444444') )

    def test_all_and_no_boundaries(self):
        '''
        Test whether all segments versus no segments produces 0.25.
        '''
        segs_a = [1,1,1,1,1,1,1,1,1,1,1,1,1]
        segs_b = [1,2,3,4,5,6,7,8,9,10,11,12,13]
        self.assertEqual(win_pr(segs_a, segs_b),
                         (2, 0, 12, 12) )
        self.assertEqual(win_pr_f(segs_a, segs_b),
                         Decimal('0.25') )
        self.assertEqual(win_pr(segs_b, segs_a),
                         (12, 72, 0, 7) )
        self.assertEqual(win_pr_f(segs_b, segs_a),
                         Decimal('0.25') )

    def test_translated_boundary(self):
        '''
        Test whether 2/3 total segments participate in mis-alignment produces
        0.75.
        '''
        segs_a = [1,1,1,1,1,2,2,2,3,3,3,3,3]
        segs_b = [1,1,1,1,2,2,2,2,3,3,3,3,3]
        self.assertEqual(win_pr(segs_a, segs_b),
                         (3, 1, 1, 21) )
        self.assertEqual(win_pr_f(segs_a, segs_b),
                         Decimal('0.75') )
        self.assertEqual(win_pr(segs_b, segs_a),
                         (3, 1, 1, 21) )
        self.assertEqual(win_pr_f(segs_b, segs_a),
                         Decimal('0.75') )
    
    def test_extra_boundary(self):
        '''
        Test whether 1/3 segments that are non-existent produces 0.889.
        '''
        segs_a = [1,1,1,1,1,2,3,3,4,4,4,4,4]
        segs_b = [1,1,1,1,1,2,2,2,3,3,3,3,3]
        self.assertEqual(win_pr(segs_a, segs_b),
                         (4, 1, 0, 21) )
        self.assertEqual(win_pr_f(segs_a, segs_b),
                         Decimal('0.8888888888888888888888888889') )
        self.assertEqual(win_pr(segs_b, segs_a),
                         (4, 0, 1, 21) )
        self.assertEqual(win_pr_f(segs_b, segs_a),
                         Decimal('0.8888888888888888888888888889') )
    
    def test_full_miss_and_misaligned(self):
        '''
        Test whether a full miss and a translated boundary out of 4 produces
        '0.6667. 
        '''
        segs_a = [1,1,1,1,1,2,3,3,4,4,4,4,4]
        segs_b = [1,1,1,1,2,2,2,2,3,3,3,3,3]
        self.assertEqual(win_pr(segs_a, segs_b),
                         (3, 2, 1, 20) )
        self.assertEqual(win_pr_f(segs_a, segs_b),
                         Decimal('0.6666666666666666666666666667') )
        self.assertEqual(win_pr(segs_b, segs_a),
                         (3, 1, 2, 20) )
        self.assertEqual(win_pr_f(segs_b, segs_a),
                         Decimal('0.6666666666666666666666666667') )
    

class TestPairwiseWinPR(unittest.TestCase):
    # pylint: disable=R0904
    '''
    Test pairwise WinPR.
    '''
    
    def test_kazantseva2012_g5(self):
        '''
        Calculate permuted pairwise percentage on Group 5 from the dataset
        collected in [KazantsevaSzpakowicz2012]_.
        '''
        self.assertEqual(pairwise_win_pr(KAZANTSEVA2012_G5, ),
                         (Decimal('0.5868367743367743367743367746'),
                          Decimal('0.1198711646320202975515773835'),
                          Decimal('0.01436909611023691387771751852'),
                          Decimal('0.02446859901846728286113130960')))
    
    def test_kazantseva2012_g2(self):
        '''
        Calculate mean permuted pairwise percentage on Group 2 from the dataset
        collected in [KazantsevaSzpakowicz2012]_.
        '''
        self.assertEqual(pairwise_win_pr(KAZANTSEVA2012_G2),
                         (Decimal('0.6980631577775708883529586185'),
                          Decimal('0.1121769660131793859585435575'),
                          Decimal('0.01258367170392200306131853234'),
                          Decimal('0.01448198403990397250690864768')))
    
    def test_large_disagreement(self):
        '''
        Calculate mean permuted pairwise percentage on a theoretical dataset
        containing large disagreement.
        '''
        self.assertEqual(pairwise_win_pr(LARGE_DISAGREEMENT),
                         (Decimal('0.2573671497584541062801932368'),
                          Decimal('0.1535469743040559128336768347'),
                          Decimal('0.02357667331793040677728768466'),
                          Decimal('0.07677348715202795641683841735')))
    
    def test_complete_agreement(self):
        '''
        Calculate mean permuted pairwise percentage on a theoretical dataset
        containing complete agreement.
        '''
        self.assertEqual(pairwise_win_pr(COMPLETE_AGREEMENT),
                         (1.0,
                          0.0,
                          0.0,
                          0.0))




'''
Segmentation similarity evaluation metric functions [FournierInkpen2012]_.

.. moduleauthor:: Chris Fournier <chris.m.fournier@gmail.com>
'''
#===============================================================================
# Copyright (c) 2011-2012, Chris Fournier
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the author nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#       
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#===============================================================================
from decimal import Decimal
from .distance.SingleBoundaryDistance import linear_edit_distance
from .. import SegmentationMetricError, compute_pairwise, \
    compute_pairwise_values, create_tsv_rows
from ..data import load_file
from ..data.TSV import write_tsv
from ..data.Display import render_mean_values


DEFAULT_N      = 2
DEFAULT_WEIGHT = (1, 1)
DEFAULT_SCALE  = True


def similarity(segment_masses_a, segment_masses_b, n=DEFAULT_N,
               weight=DEFAULT_WEIGHT, scale_transp=DEFAULT_SCALE,
               return_parts=False):
    # pylint: disable=C0103,R0913,R0914
    '''
    Calculates similarity between two sequences of segment masses using
    boundary edit distance as in [FournierInkpen2012]_.
    
    .. math::
        \\text{S}(s_{i1},s_{i2}) = \\frac
        {\\textbf{t} \cdot \\text{mass}(i) - \\textbf{t} - \\text{d}(s_{i1},s_{i2},T)}
        {\\textbf{t} \cdot \\text{mass}(i) - \\textbf{t}}
    
    :param segment_masses_a:  Segmentation masses.
    :param segment_masses_b:  Segmentation masses.
    :param n:                 The maximum number of PBs that boundaries can \
                                  span to be considered transpositions (n<2 \
                                  means no transpositions)
    :param weight:            Weights for substitution and transposition \
                                  operations
    :param scale_transp:      If true, scales transpositions by their size \
                                  and the number of boundaries
    :param return_parts:      Scales how precision and recall are averaged.
    :type segment_masses_a: list
    :type segment_masses_b: list
    :type n: int
    :type weight: :func:`tuple` of :func:`float` objects \
                        (substitutions,transpositions)
    :type scale_transp: bool
    :type return_parts: bool
    
    :returns: Similarity, where 0.0 <= sim <= 1.0, or the pbs unedited, total \
        pbs, substitutions and transpositions
    :rtype: :class:`decimal.Decimal`, or :func:`int`, :func:`int`, \
        :func:`int`, :func:`int`
    '''
    # Total number of segments to be evaluated
    if len(segment_masses_a) == 0 and len(segment_masses_b) == 0:
        if return_parts:
            return 0, 0
        else:
            return Decimal('1.0')
    elif sum(segment_masses_a) != sum(segment_masses_b):
        raise SegmentationMetricError('Unequal segmentation masses (%i != %i)' \
                                      % (sum(segment_masses_a),
                                         sum(segment_masses_b)))
    elif 0 in segment_masses_a or 0 in segment_masses_b:
        raise SegmentationMetricError('Non-plausable mass 0 present')
    else:
        # Compute total pbs
        pbs_total = sum(segment_masses_a) - 1
        
        # Compute edit distance
        set_transpositions, set_errors = \
            linear_edit_distance(segment_masses_a, segment_masses_b, n)[1:3]
        
        # Get totals
        total_set_transpositions = len(set_transpositions)
        total_set_errors         = len(set_errors)
        
        # Scale transpositions
        if scale_transp:
            total_set_transpositions = 0
            for set_transposition in set_transpositions:
                total_set_transpositions += set_transposition.te()
        
        # Apply weights
        if weight != DEFAULT_WEIGHT:
            weight_s = Decimal(weight[0])
            weight_t = Decimal(weight[1])
            total_set_errors         = Decimal(total_set_errors) * weight_s
            total_set_transpositions = Decimal(total_set_transpositions) * \
                                            weight_t
        
        pbs_unedited = pbs_total - (total_set_errors + total_set_transpositions)
        
        if return_parts:
            # Return the total sum of unmoved mass during all transformations,
            # and the total mass
            return pbs_unedited, pbs_total, total_set_errors, \
                total_set_transpositions, set_errors, set_transpositions
        else:
            # Return the total sum of unmoved mass during all transformations \
            # over the total mass
            return Decimal(pbs_unedited) / Decimal(pbs_total)


def pairwise_similarity(dataset_masses, n=DEFAULT_N, weight=DEFAULT_WEIGHT,
                        scale_transp=DEFAULT_SCALE, return_parts=False):
    '''
    Calculate mean pairwise segmentation F-Measure.
    
    .. seealso:: :func:`similarity`
    .. seealso:: :func:`segeval.compute_pairwise`
    
    :param dataset_masses: Segmentation mass dataset (including multiple \
                           codings).
    :type dataset_masses: dict
        
    :returns: Mean, standard deviation, variance, and standard error of a \
        segmentation metric.
    :rtype: :class:`decimal.Decimal`, :class:`decimal.Decimal`, \
        :class:`decimal.Decimal`, :class:`decimal.Decimal`
    '''
    # pylint: disable=C0103,R0913,R0914
    def wrapper(segment_masses_a, segment_masses_b, return_parts=return_parts):
        '''
        Wrapper to provide parameters.
        '''
        return similarity(segment_masses_a, segment_masses_b, n, weight,
                  scale_transp, return_parts)
    
    return compute_pairwise(dataset_masses, wrapper, permuted=False)


OUTPUT_NAME = 'Mean S'
SHORT_NAME  = 'S'


def values_s_detailed(dataset_masses, n=DEFAULT_N, weight=DEFAULT_WEIGHT,
                      scale_transp=DEFAULT_SCALE):
    '''
    Produces a TSV for this metric
    '''
    # pylint: disable=C0103
    header = list(['coder1', 'coder2', 'error', 'edits', 'boundaries', 'n'])
    def wrapper(segment_masses_a, segment_masses_b, return_parts):
        '''
        Wrapper to provide parameters.
        '''
        return similarity(segment_masses_a, segment_masses_b, n, weight,
                  scale_transp, return_parts)
    # Get values
    values = compute_pairwise_values(dataset_masses, wrapper, return_parts=True)
    adjusted_values = dict()
    for label, value in values.items():
        subvalues = list()
        set_errors, set_transpositions = value[4:6]
        # For set errors
        for _ in set_errors:
            subvalues.append(['sub', 1, 1, 1])
        # For transposition errors
        for set_transposition in set_transpositions:
            subvalues.append(['transp', 1, set_transposition.boundaries,
                             set_transposition.n])
        adjusted_values[label] = subvalues
    return create_tsv_rows(header, adjusted_values, expand=True)


def values_s(dataset_masses, n=DEFAULT_N, weight=DEFAULT_WEIGHT,
                      scale_transp=DEFAULT_SCALE):
    '''
    Produces a TSV for this metric
    '''
    # pylint: disable=C0103
    header = list(['coder1', 'coder2', 'pbs_unedited', 'pbs_total', \
                   'sub_edits', 'transp_edits', SHORT_NAME])
    def wrapper(segment_masses_a, segment_masses_b, return_parts):
        '''
        Wrapper to provide parameters.
        '''
        return similarity(segment_masses_a, segment_masses_b, n, weight,
                  scale_transp, return_parts)
    # Get values
    values = compute_pairwise_values(dataset_masses, wrapper, return_parts=True)
    adjusted_values = dict()
    for label, value in values.items():
        # Get values
        pbs_unedited, pbs_total, total_set_errors, \
                total_set_transpositions = value[0:4]
        s = Decimal(pbs_unedited) / Decimal(pbs_total)
        # Store values
        adjusted_values[label] = [pbs_unedited, pbs_total, total_set_errors, \
                total_set_transpositions, s]
    return create_tsv_rows(header, adjusted_values)


def parse(args):
    '''
    Parse this module's metric arguments and perform requested actions.
    '''
    # pylint: disable=C0103
    output = None
    values = load_file(args)[0]
    # Parse args
    n  = args['n']
    wt = args['wt']
    ws = args['ws']
    te = args['te']
    weight = DEFAULT_WEIGHT
    if wt != 1.0 or ws != 1.0:
        weight = (ws, wt)
    # Is a TSV requested?
    if args['output'] != None:
        # Create a TSV
        output_file = args['output'][0]
        if args['detailed']:
            header, rows = values_s_detailed(values, n, weight, te)
        else:
            header, rows = values_s(values, n, weight, te)
        write_tsv(output_file, header, rows)
    else:
        # Create a string to output
        mean, std, var, stderr = pairwise_similarity(values, n, weight, te)
        output = render_mean_values(SHORT_NAME, mean, std, var, stderr)
    # Return
    return output


def parser_s_support(parser):
    '''
    Add support for S parameters
    
    :param parser: Argument parser
    :type parser: argparse.ArgumentParser
    '''
    parser.add_argument('-n',
                        type=int,
                        default=DEFAULT_N,
                        help='The maximum number of PBs that boundaries can '+\
                              'span to be considered transpositions (n<2 '+\
                              ('means no transpositions); default is %i.' \
                                    % DEFAULT_N))
    parser.add_argument('-wt',
                        type=float,
                        default=DEFAULT_WEIGHT[0],
                        help='Weight, 0 <= wt <= 1, to scale transposition '+\
                            'error by; default is 1 (no scaling).')
    parser.add_argument('-ws',
                        type=float,
                        default=DEFAULT_WEIGHT[0],
                        help='Weight, 0 <= wt <= 1, to scale substitution '+\
                            'error by; default is 1 (no scaling).')
    parser.add_argument('-te',
                        type=bool,
                        default=DEFAULT_SCALE,
                        help='Scale transpositions by their size and the '+\
                            'number of boundaries the span; %s by default' \
                            % str(DEFAULT_SCALE))


def create_parser(subparsers):
    '''
    Setup a command line parser for this module's metric.
    '''
    from ..data import parser_add_file_support
    parser = subparsers.add_parser('s',
                                   help=OUTPUT_NAME)
    parser_add_file_support(parser)
    parser_s_support(parser)
    parser.add_argument('-de', '--detailed',
                        action='store_true',
                        default=False,
                        help='When specifying an output TSV file, specify '+\
                            'this to obtain a detailed error breakdown per '+\
                            'edit')
    parser.set_defaults(func=parse)




'''
Tests some general segeval utility functions.

.. moduleauthor:: Chris Fournier <chris.m.fournier@gmail.com>
'''
#===============================================================================
# Copyright (c) 2012, Chris Fournier
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the author nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#       
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#===============================================================================
import unittest
from decimal import Decimal
from . import convert_positions_to_masses, convert_masses_to_positions, \
    compute_pairwise, compute_mean
from .data.Samples import KAZANTSEVA2012_G5
from .ml.FbMeasure import f_b_measure
from .agreement.Kappa import fleiss_kappa


class TestSegeval(unittest.TestCase):
    '''
    segeval utility function tests.
    '''
    #pylint: disable=R0904,C0103
    
    def test_convert_positions_to_masses(self):
        '''
        Test segment position sequence conversion to masses.
        '''
        #pylint: disable=C0324
        self.assertEqual([5,3,5],
                         convert_positions_to_masses(
                            [1,1,1,1,1,2,2,2,3,3,3,3,3]))
        
      
    def test_convert_masses_to_positions(self):
        '''
        Test segment position sequence conversion to masses.
        '''
        #pylint: disable=C0324
        self.assertEqual([1,1,1,1,1,2,2,2,3,3,3,3,3],
                         convert_masses_to_positions([5,3,5]))
    
    
    def test_compute_pairwise(self):
        '''
        Tests computing pairwise values from dicts.  Ensures that dicts of
        arbitrary size can still be used.
        '''
        expected = (Decimal('0.2441586812212541867438777318'),
                    Decimal('0.2305169230001435997031211479'),
                    Decimal('0.05313805178945413332341442162'),
                    Decimal('0.04705406986889928477977431110'))
        expected_variant = (Decimal('0.2441586812212541867438777318'),
                            Decimal('0.2305169230001435997031211480'),
                            Decimal('0.05313805178945413332341442167'),
                            Decimal('0.04705406986889928477977431112'))
        
        self.assertEqual(compute_pairwise(KAZANTSEVA2012_G5,
                                          f_b_measure),
                         expected)
        self.assertEqual(compute_pairwise({'G5' : KAZANTSEVA2012_G5},
                                          f_b_measure),
                         expected)
        self.assertEqual(compute_pairwise({'Kazantseva' : 
                                                {'G5' : KAZANTSEVA2012_G5}},
                                          f_b_measure),
                         expected_variant)
    
    
    def test_compute_mean(self):
        '''
        Tests computing pairwise values from dicts.  Ensures that dicts of
        arbitrary size can still be used.
        '''
        expected = ((Decimal('0.8198449828922561273654074438'),
                     Decimal('2E-28'),
                     Decimal('4E-56'),
                     Decimal('1E-28')))
        
        self.assertEqual(compute_mean(KAZANTSEVA2012_G5, fleiss_kappa),
                         expected)
        self.assertEqual(compute_mean({'G5' : KAZANTSEVA2012_G5}, fleiss_kappa),
                         expected)
        self.assertEqual(compute_mean({'Kazantseva' : 
                                            {'G5' : KAZANTSEVA2012_G5}},
                                                fleiss_kappa),
                         expected)
        
        


'''
Segmentation evaluation metric package. Provides evaluation metrics to
evaluate the performance of both human and automatic text (i.e., discourse)
segmenters.  This package contains a new metric called Segmentation Similarity
(S) [FournierInkpen2012]_ which is recommended for usage along with a variety
of inter-coder agreement coefficients that utilize S.

To use S, see the :mod:`segeval.similarity` module.

.. moduleauthor:: Chris Fournier <chris.m.fournier@gmail.com>
'''
#===============================================================================
# Copyright (c) 2011-2012, Chris Fournier
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the author nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#       
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#===============================================================================
import os
from decimal import Decimal
from collections import Counter
from .Math import mean, std, var, stderr

ROOT_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                        os.sep.join(['..'] * 1)))

DEBUG_MODE  = False
EXPERIMENTS = False


def load_tests(loader, tests, pattern):
    '''
    A ``load_tests()`` function utilizing the default loader
    :func:`segeval.Utils.default_load_tests`.
    
    .. seealso:: The `load_tests protocol <http://docs.python.org/library/\
    unittest.html#load-tests-protocol>`_.
    '''
    #pylint: disable=W0613
    from Utils import default_load_tests
    return default_load_tests(__file__, loader, tests)


def convert_positions_to_masses(positions):
    '''
    Convert an ordered sequence of boundary position labels into a
    sequence of segment masses, e.g., ``[1,1,1,1,1,2,2,2,3,3,3,3,3]`` becomes
    ``[5,3,5]``.
    
    :param segments: Ordered sequence of which segments a unit belongs to.
    :type segments: list
    
    :returns: Segment mass sequence.
    :rtype: :func:`list`
    
    .. deprecated:: 1.0
    '''
    counts = Counter(positions)
    masses = list()
    for i in range(1, max(counts.keys()) + 1):
        masses.append(counts[i])
    return masses


def convert_masses_to_positions(masses):
    '''
    Converts a sequence of segment masses into an ordered sequence of section
    labels for each unit, e.g., ``[5,3,5]`` becomes
    ``[1,1,1,1,1,2,2,2,3,3,3,3,3]``.
    
    :param masses: Segment mass sequence.
    :type masses: list
    
    :returns: Ordered sequence of which segments a unit belongs to.
    :rtype: :func:`list`
    '''
    sequence = list()
    for i, mass in enumerate(masses):
        sequence.extend([i + 1] * mass)
    return sequence


def compute_pairwise(dataset_masses, fnc_metric, permuted=False):
    '''
    Calculate mean pairwise segmentation metric pairs for functions that take
    pairs of segmentations.
    
    :param dataset_masses: Segmentation mass dataset (including multiple \
                           codings).
    :param fnc_metric:     Metric function to call on segmentation mass pairs.
    :param permuted:       Permute coder combinations if true.
    :type dataset_masses: dict
    :type fnc_metric:     func
    :type permuted:       bool
    
    :returns: |compute_mean_return|
    :rtype: |compute_mean_return_type|
    '''
    pairs = compute_pairwise_values(dataset_masses, fnc_metric, permuted)
    return mean(pairs.values()), std(pairs.values()), var(pairs.values()), \
        stderr(pairs.values())


def compute_pairwise_values(dataset_masses, fnc_metric, permuted=False,
                            return_parts=False):
    '''
    Calculate mean pairwise segmentation metric pairs for functions that take
    pairs of segmentations.
    
    :param dataset_masses: Segmentation mass dataset (including multiple \
                           codings).
    :param fnc_metric:     Metric function to call on segmentation mass pairs.
    :param permuted:       Permute coder combinations if true.
    :type dataset_masses: dict
    :type fnc_metric:     func
    :type permuted:       bool
    
    :returns: List of values
    :rtype: :func:`list`
    '''
    # pylint: disable=C0103
    pairs = dict()
    # Define fnc per group
    def __per_group__(prefix, inner_dataset_masses):
        '''
        Recurse through a dict to find levels where a metric can be calculated.
        
        
        :param inner_dataset_masses: Segmentation mass dataset (including \
                                     multiple codings).
        :type inner_dataset_masses: dict
        '''
        for label, coder_masses in inner_dataset_masses.items():
            if len(coder_masses.values()) > 0 and \
                isinstance(coder_masses.values()[0], list):
                # If is a group
                coders = coder_masses.keys()
                for m in range(0, len(coders)):
                    for n in range(m+1, len(coders)):
                        segs_m = coder_masses[coders[m]]
                        segs_n = coder_masses[coders[n]]
                        entry_parts = list(prefix)
                        entry_parts.extend([label, str(m), str(n)])
                        entry = ','.join(entry_parts)
                        if return_parts:
                            pairs[entry] = \
                                fnc_metric(segs_m, segs_n,
                                                   return_parts=return_parts)
                        else:
                            pairs[entry] = \
                                Decimal(fnc_metric(segs_m, segs_n))
                            
                        if permuted:
                            entry_parts = list(prefix)
                            entry_parts.extend([label, str(n), str(m)])
                            entry = ','.join(entry_parts)
                            if return_parts:
                                pairs[entry] = \
                                    fnc_metric(segs_n, segs_m,
                                                    return_parts=return_parts)
                            else:
                                pairs[entry] = \
                                    Decimal(fnc_metric(segs_n, segs_m))
            else:
                # Else, recurse deeper
                innter_prefix = list(prefix)
                innter_prefix.append(label)
                __per_group__(innter_prefix, coder_masses)
    # Parse
    __per_group__(list(), dataset_masses)
    # Return mean, std dev, and variance
    return pairs


def compute_mean(dataset_masses, fnc_metric):
    '''
    Calculate mean segmentation metric values for functions that take
    dicts of items and their segmentations per coder (``items_masses``).
    
    .. seealso:: :func:`segeval.agreement.observed_agreement` for an example of\
     ``items_masses``.
    
    :param dataset_masses: Segmentation mass dataset (including multiple \
                           codings).
    :param fnc_metric:     Metric function to call on segmentation mass pairs.
    :type dataset_masses: dict
    :type fnc_metric:     func
    
    .. |compute_mean_return| replace:: Mean, standard deviation, variance, and \
        standard error of a segmentation metric.
    .. |compute_mean_return_type| replace:: :class:`decimal.Decimal`, \
        :class:`decimal.Decimal`, :class:`decimal.Decimal`, \
        :class:`decimal.Decimal`
        
    :returns: |compute_mean_return|
    :rtype: |compute_mean_return_type|
    '''
    # pylint: disable=C0103
    values = compute_mean_values(dataset_masses, fnc_metric)
    # Return mean, std dev, and variance
    return mean(values.values()), std(values.values()), var(values.values()), \
                stderr(values.values())


def compute_mean_values(dataset_masses, fnc_metric):
    '''
    Calculate mean segmentation metric values for functions that take
    dicts of items and their segmentations per coder (``items_masses``).
    
    .. seealso:: :func:`segeval.agreement.observed_agreement` for an example of\
     ``items_masses``.
    
    :param dataset_masses: Segmentation mass dataset (including multiple \
                           codings).
    :param fnc_metric:     Metric function to call on segmentation mass pairs.
    :type dataset_masses: dict
    :type fnc_metric:     func
    
    .. |compute_mean_return| replace:: Mean, standard deviation, variance, and \
        standard error of a segmentation metric.
    .. |compute_mean_return_type| replace:: :class:`decimal.Decimal`, \
        :class:`decimal.Decimal`, :class:`decimal.Decimal`, \
        :class:`decimal.Decimal`
        
    :returns: |compute_mean_return|
    :rtype: |compute_mean_return_type|
    '''
    # pylint: disable=C0103
    values = dict()
    # Define fnc per group
    def __per_group__(prefix, inner_dataset_masses):
        '''
        Recurse through a dict to find levels where a metric can be calculated.
        
        
        :param inner_dataset_masses: Segmentation mass dataset (including \
                                     multiple codings).
        :type inner_dataset_masses: dict
        '''
        for label, coder_masses in inner_dataset_masses.items():
            if isinstance(coder_masses, dict) and \
                len(coder_masses.values()) > 0 and \
                isinstance(coder_masses.values()[0], list):
                # If is a group
                entry_parts = list(prefix)
                entry_parts.append(label)
                values[','.join(entry_parts)] = fnc_metric(inner_dataset_masses)
            else:
                # Else, recurse deeper
                inner_prefix = list(prefix)
                inner_prefix.append(label)
                __per_group__(inner_prefix, coder_masses)
    # Parse
    __per_group__(list(), dataset_masses)
    # Return mean, std dev, and variance
    return values


def create_tsv_rows(header, values, expand=False):
    '''
    Convert a dict of values into a list of properly padded rows.
    
    :param filepath: Path and filename of a file to write to
    :param header:   List of known category names
    :param values:   Dict of computed values
    :type header: :class:`list`
    :type values: :class:`dict`
    '''
    # pylint: disable=R0914
    # Parse labels
    rows = list()
    max_len = 0
    for key, value in values.items():
        # Get label parts
        items_parts = key.split(',')
        # Expand if we are creating multiple rows per label part
        subvalues = None
        if expand:
            subvalues = value
        else:
            subvalues = [value]
        # Create rows
        for subvalue in subvalues:
            row = list(items_parts)
            if isinstance(subvalue, list):
                row.extend(subvalue)
            else:
                row.append(subvalue)
            rows.append(row)
            max_len = len(row) if len(row) > max_len else max_len
    # Pad rows to match the max depth/number of labels
    padded_rows = list()
    for row in rows:
        difference = max_len - len(row)
        if difference > 0:
            padded_row = list([''] * difference)
            padded_row.extend(row)
            padded_rows.append(padded_row)
        else:
            padded_rows.append(row)
    # Pad headers to match the depth/number of labels
    labels = max_len - len(header)
    padded_header = ['label%i' % i for i in xrange(1, labels + 1)]
    padded_header.extend(header)
    # Return
    return padded_header, padded_rows


class SegmentationMetricError(Exception):
    '''
    Indicates that a runtime check has failed, and the algorithm is performing
    incorrectly, or input validation has failed.  Generation of this exception
    is tested.
        
    :param message: Explanation for the exception.
    :type message: str
    '''
    
    def __init__(self, message):
        '''
        Initializer.
        
        :param message: Explanation for the exception.
        :type message: str
        '''
        Exception.__init__(self, message)




'''
Implementation of the Pk segmentation evaluation metric described in 
[BeefermanBerger1999]_

@author: Chris Fournier
@contact: chris.m.fournier@gmail.com
'''
#===============================================================================
# Copyright (c) 2011-2012, Chris Fournier
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the author nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#       
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#===============================================================================
from decimal import Decimal
from . import compute_window_size, parser_one_minus_support
from .. import SegmentationMetricError, compute_pairwise, \
    convert_masses_to_positions, compute_pairwise_values, create_tsv_rows
from ..data import load_file
from ..data.TSV import write_tsv
from ..data.Display import render_mean_values


def pk(hypothesis_positions, reference_positions, window_size=None,
       one_minus=False, convert_from_masses=False):
    '''
    Calculates the Pk segmentation evaluation metric score for a
    hypothetical segmentation against a reference segmentation for a given
    window size.  The standard method of calculating the window size
    is performed if a window size is not specified.
    
    :param hypothesis_positions: Hypothesis segmentation section labels
                                    sequence.
    :param reference_positions:  Reference segmentation section labels sequence.
    :param window_size:          The size of the window that is slid over the \
                                    two segmentations used to count mismatches \
                                    (default is None and will use the average \
                                    window size)
    :param one_minus:            Return 1-Pk to make it no longer a \
                                    penalty-metric.
    :param convert_from_masses:  Convert the segmentations provided from \
                                    masses into positions.
    :type hypothesis_positions: list
    :type reference_positions: list
    :type window_size: int
    :type one_minus: bool
    :type convert_from_masses: bool
    
    .. note:: See :func:`segeval.convert_masses_to_positions` for an example of
              the input format.
    '''
    # pylint: disable=C0103
    # Convert from masses into positions 
    if convert_from_masses:
        reference_positions  = convert_masses_to_positions(reference_positions)
        hypothesis_positions = convert_masses_to_positions(hypothesis_positions)
    # Check for input errors
    if len(reference_positions) != len(hypothesis_positions):
        raise SegmentationMetricError(
                    'Reference and hypothesis segmentations differ in length.')
    # Compute window size to use if unspecified
    if window_size is None:
        window_size = compute_window_size(reference_positions)
    # Create a set of pairs of units from each segmentation to go over using a
    # window
    sum_differences = 0
    # Slide window over and sum the number of varying windows
    measurements = 0
    for i in xrange(0, len(reference_positions) - (window_size)):
        # Create probe windows with k boundaries inside
        window_ref = reference_positions[i:i+window_size+1]
        window_hyp = hypothesis_positions[i:i+window_size+1]
        # Probe agreement
        agree_ref = window_ref[0] == window_ref[-1]
        agree_hyp = window_hyp[0] == window_hyp[-1]
        # If the windows agreements agree
        if agree_ref != agree_hyp:
            sum_differences += 1
        measurements += 1
    # Perform final division
    p_k = Decimal(sum_differences) / measurements
    if not one_minus:
        return p_k
    else:
        return Decimal('1.0') - p_k


def pairwise_pk(dataset_masses, one_minus=False, convert_from_masses=True):
    '''
    Calculate mean pairwise segmentation F-Measure.
    
    .. seealso:: :func:`pk`
    .. seealso:: :func:`segeval.compute_pairwise`
    
    :param dataset_masses: Segmentation mass dataset (including multiple \
                           codings).
    :type dataset_masses: dict
        
    :returns: Mean, standard deviation, variance, and standard error of a \
        segmentation metric.
    :rtype: :class:`decimal.Decimal`, :class:`decimal.Decimal`, \
        :class:`decimal.Decimal`, :class:`decimal.Decimal`
    '''
    def wrapper(hypothesis_masses, reference_masses):
        '''
        Wrapper to provide parameters.
        '''
        return pk(hypothesis_masses, reference_masses, one_minus=one_minus,
                  convert_from_masses=convert_from_masses)
    
    return compute_pairwise(dataset_masses, wrapper, permuted=True)


OUTPUT_NAME = 'Mean Pk'
SHORT_NAME  = 'Pk'


def values_pk(dataset_masses, name, one_minus):
    '''
    Produces a TSV for this metric
    '''
    # Define a fnc to pass parameters
    def wrapper(hypothesis_masses, reference_masses):
        '''
        Wrapper to provide parameters.
        '''
        return pk(hypothesis_masses, reference_masses, one_minus=one_minus,
                  convert_from_masses=True)
    # Get values
    header = list(['coder1', 'coder2', name])
    values = compute_pairwise_values(dataset_masses, wrapper, permuted=True)
    return create_tsv_rows(header, values)


def parse(args):
    '''
    Parse this module's metric arguments and perform requested actions.
    '''
    output = None
    values = load_file(args)[0]
    one_minus = args['oneminus']
    name = SHORT_NAME
    
    if one_minus:
        name = '1 - %s' % name
    
    # Is a TSV requested?
    if args['output'] != None:
        # Create a TSV
        output_file = args['output'][0]
        header, rows = values_pk(values, name, one_minus)
        write_tsv(output_file, header, rows)
    else:
        # Create a string to output
        mean, std, var, stderr = pairwise_pk(values, one_minus)
        output = render_mean_values(name, mean, std, var, stderr)
    return output


def create_parser(subparsers):
    '''
    Setup a command line parser for this module's metric.
    '''
    from ..data import parser_add_file_support
    parser = subparsers.add_parser('pk',
                                   help=OUTPUT_NAME)
    parser_add_file_support(parser)
    parser_one_minus_support(parser)
    parser.set_defaults(func=parse)




'''
Implementation of the WinPR segmentation evaluation metric described in Scaiano
and Inkpen (2012).

.. moduleauthor:: Chris Fournier <chris.m.fournier@gmail.com>
'''
#===============================================================================
# Copyright (c) 2011-2012, Chris Fournier
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the author nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#       
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#===============================================================================
from .WindowDiff import compute_window_size, create_paired_window
from ..ml import fmeasure, precision, recall
from ..ml.FbMeasure import parser_beta_support, DEFAULT_BETA
from .. import SegmentationMetricError, compute_pairwise, \
    convert_masses_to_positions, compute_pairwise_values, create_tsv_rows
from ..data import load_file
from ..data.TSV import write_tsv
from ..data.Display import render_mean_values


def win_pr(hypothesis_positions, reference_positions, window_size=None,
           convert_from_masses=False):
    '''
    Calculates the WinPR segmentation evaluation metric confusion matrix for a
    hypothetical segmentation against a reference segmentation for a given
    window size.  The standard WindowDiff method of calculating the window size
    is performed when a window size is not specified.
    
    :param hypothesis_positions:     Hypothesis segmentation section labels
                                        sequence.
    :param reference_positions:      Reference segmentation section labels
                                        sequence.
    :param window_size:              The size of the window that is slid over \
                                        the two segmentations used to count \
                                        mismatches (default is None and will \
                                        use the average window size)
    :param return_fscore:            If True, specifies that an F-score is to \
                                        be returned.
    :param convert_from_masses:      Convert the segmentations provided from \
                                        masses into positions.
    :type hypothesis_positions: list
    :type reference_positions: list
    :type window_size: int
    :type convert_from_masses: bool
    
    :returns: TP, FP, FN, TN
    :rtype: :func:`int`, :func:`int`, :func:`int`, :func:`int`
    '''
    # pylint: disable=C0103, R0914
    # Convert from masses into positions 
    if convert_from_masses:
        reference_positions  = convert_masses_to_positions(reference_positions)
        hypothesis_positions = convert_masses_to_positions(hypothesis_positions)
    # Check for input errors
    if len(reference_positions) != len(hypothesis_positions):
        raise SegmentationMetricError(
                    'Reference and hypothesis segmentations differ in length.')
    # Compute window size to use if unspecified
    if window_size is None:
        window_size = compute_window_size(reference_positions)
    # Create a set of pairs of units from each segmentation to go over using a
    # window
    tp, fp, fn = [0] * 3
    tn = (-1 * window_size) * (window_size - 1)
    # Create and append phantom boundaries at the beginning and end of the
    # segmentation to properly count boundaries at the beginning and end
    units_ref_hyp = create_paired_window(hypothesis_positions,
                                         reference_positions, window_size,
                                         True)[0]
    # Slide window over and calculate TP, TN, FP, FN
    for i in xrange(0, len(units_ref_hyp) - window_size + 1):
        window = units_ref_hyp[i:i + window_size]
        ref_boundaries = 0
        hyp_boundaries = 0
        # For pair in window
        for j in xrange(0, len(window) - 1):
            ref_part, hyp_part = zip(*window[j:j + 2])
            # Boundary exists in the reference segmentation
            if ref_part[0] != ref_part[1]:
                ref_boundaries += 1
            # Boundary exists in the hypothesis segmentation
            if hyp_part[0] != hyp_part[1]:
                hyp_boundaries += 1
        # If the number of boundaries per segmentation in the window differs
        tp += min(ref_boundaries, hyp_boundaries)
        fp += max(0, hyp_boundaries - ref_boundaries)
        fn += max(0, ref_boundaries - hyp_boundaries)
        tn += (window_size - max(ref_boundaries, hyp_boundaries))
    # Return the constituent statistics
    return tp, fp, fn, tn


def win_pr_f(hypothesis_positions, reference_positions, window_size=None,
             convert_from_masses=False, beta=DEFAULT_BETA):
    '''
    Compute F_beta Measure from WinPR.
    
    .. seealso:: :func:`win_pr`
    .. seealso:: :func:`segeval.ml.fmeasure`
    '''
    # pylint: disable=C0103
    tp, fp, fn = win_pr(hypothesis_positions, reference_positions,
                            window_size, convert_from_masses)[0:3]
    return fmeasure(tp, fp, fn, beta)


def wrap_win_p_f(beta=DEFAULT_BETA):
    '''
    Create a wrapper for win_pr_f
    '''
    def wrapper(hypothesis_masses, reference_masses, window_size,
                convert_from_masses):
        '''
        Wrapper to provide the beta parameter.
        '''
        return win_pr_f(hypothesis_masses, reference_masses, window_size,
                        convert_from_masses, beta=beta)
    return wrapper


def win_pr_p(hypothesis_positions, reference_positions, window_size=None,
             convert_from_masses=False):
    '''
    Compute Precision from WinPR.
    
    .. seealso:: :func:`win_pr`
    .. seealso:: :func:`segeval.ml.precision`
    '''
    # pylint: disable=C0103
    tp, fp = win_pr(hypothesis_positions, reference_positions, window_size,
                    convert_from_masses)[0:2]
    return precision(tp, fp)


def win_pr_r(hypothesis_positions, reference_positions, window_size=None,
             convert_from_masses=False):
    '''
    Compute Recall from WinPR.
    
    .. seealso:: :func:`win_pr`
    .. seealso:: :func:`segeval.ml.recall`
    '''
    # pylint: disable=C0103
    values = win_pr(hypothesis_positions, reference_positions, window_size,
                    convert_from_masses)
    tp = values[0]
    fn = values[2]
    return recall(tp, fn)


def pairwise_win_pr(dataset_masses,
                    fnc_winpr=win_pr_f,
                    window_size=None,
                    convert_from_masses=True):
    '''
    Calculate mean pairwise WinPR-F_beta, WinPR-P, or WinPR-R.
    
    .. seealso:: :func:`win_pr`
    .. seealso:: :func:`segeval.compute_pairwise`
    
    :param dataset_masses: Segmentation mass dataset (including multiple \
                           codings).
    :type dataset_masses: dict
        
    :returns: Mean, standard deviation, variance, and standard error of a \
        segmentation metric.
    :rtype: :class:`decimal.Decimal`, :class:`decimal.Decimal`, \
        :class:`decimal.Decimal`, :class:`decimal.Decimal`
    '''
    def wrapper(hypothesis_masses, reference_masses):
        '''
        Wrapper to provide parameters.
        '''
        return fnc_winpr(hypothesis_masses, reference_masses, window_size,
                         convert_from_masses)
    
    return compute_pairwise(dataset_masses, wrapper, permuted=False)


OUTPUT_NAME = 'Mean WinPR'
SHORT_NAME  = 'WinPR-%s'
SHORT_NAME_F = 'f'
SHORT_NAME_P = 'p'
SHORT_NAME_R = 'r'


def values_win_pr(dataset_masses, beta=DEFAULT_BETA):
    '''
    Produces a TSV for this metric
    '''
    # pylint: disable=C0103
    # Define a fnc to retrieve F_Beta-Measure values
    def wrapper_winpr(hypothesis_masses, reference_masses, return_parts):
        '''
        Wrapper to provide parameters.
        '''
        # pylint: disable=W0613
        return win_pr(hypothesis_masses, reference_masses,
                      convert_from_masses=True)
    # Create header
    header = list(['coder1', 'coder2',
                   SHORT_NAME % SHORT_NAME_F + '_' + str(beta),
                   SHORT_NAME_P, SHORT_NAME_R, 'TP', 'FP', 'FN', 'TN'])
    # Calculate values
    values_cf = compute_pairwise_values(dataset_masses, wrapper_winpr,
                                       permuted=False, return_parts=True)
    # Combine into one table
    combined_values = dict()
    for label, cf in values_cf.items():
        tp, fp, fn = cf[0:3]
        
        row = list()
        row.append(fmeasure(tp, fp, fn, beta))
        row.append(precision(tp, fp))
        row.append(recall(tp, fn))
        row.extend(cf)
        
        combined_values[label] = row
    # Return
    return create_tsv_rows(header, combined_values)


def parse(args):
    '''
    Parse this module's metric arguments and perform requested actions.
    '''
    output = None
    values = load_file(args)[0]
    subsubparser_name = args['subsubparser_name']
    name = SHORT_NAME % subsubparser_name
    beta = 1
    if 'beta' in args and args['beta'] != 1:
        beta = args['beta']
    # Is a TSV requested?
    if args['output'] != None:
        # Create a TSV
        output_file = args['output'][0]
        header, rows = values_win_pr(values, beta)
        write_tsv(output_file, header, rows)
    else:
        # Create a string to output
        if subsubparser_name == SHORT_NAME_F:
            name += '_%s' % str(beta)
            mean, std, var, stderr = pairwise_win_pr(values, wrap_win_p_f(beta))
        elif subsubparser_name == SHORT_NAME_P:
            mean, std, var, stderr = pairwise_win_pr(values, win_pr_p)
        elif subsubparser_name == SHORT_NAME_R:
            mean, std, var, stderr = pairwise_win_pr(values, win_pr_r)
        output = render_mean_values(name, mean, std, var, stderr)
    # Return
    return output

    
def create_submetric_parser(subparsers):
    '''
    Setup a command line parser for this module's sub metrics.
    '''
    from ..data import parser_add_file_support
    
    parser_f = subparsers.add_parser(SHORT_NAME_F, help='F_beta Measure')
    parser_beta_support(parser_f)
    parser_add_file_support(parser_f)
    parser_f.set_defaults(func=parse)
    
    parser_r = subparsers.add_parser(SHORT_NAME_R, help='Recall')
    parser_add_file_support(parser_r)
    parser_r.set_defaults(func=parse)
    
    parser_p = subparsers.add_parser(SHORT_NAME_P, help='Precision')
    parser_add_file_support(parser_p)
    parser_p.set_defaults(func=parse)
    

def create_parser(subparsers):
    '''
    Setup a command line parser for this module's metric.
    '''
    parser = subparsers.add_parser('wpr',
                                   help=OUTPUT_NAME)
    
    subsubparsers = parser.add_subparsers(title='submetric', 
                                       description='Calculates a specified '+\
                                            'information retrieval (IR) '+\
                                            'metric from values provided by '+\
                                            'WinPR',
                                       help='Available IR metrics',
                                       dest='subsubparser_name')
    create_submetric_parser(subsubparsers)




'''
Setup script for installing the segeval package.

.. codeauthor:: Chris Fournier <chris.m.fournier@gmail.com>
'''
from distutils.core import setup
setup(name='segeval',
      version='1.0.1 beta',
      
      description='A package and utilities providing a variety of discourse \
segmentation evaluation metrics',
      
      author='Chris Fournier',
      author_email='chris.m.fournier@gmail.com',
      
      maintainer='Chris Fournier',
      maintainer_email='chris.m.fournier@gmail.com',
      
      url='http://nlp.chrisfournier.ca/software/',
      download_url = 'http://pypi.python.org/pypi/segeval/',
      
      classifiers=['Development Status :: 4 - Beta',
                   'Environment :: Console',
                   'Intended Audience :: Developers',
                   'Intended Audience :: Science/Research',
                   'License :: OSI Approved :: BSD License',
                   'Natural Language :: English',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Topic :: Scientific/Engineering :: Artificial Intelligence',
                   'Topic :: Scientific/Engineering :: Information Analysis',
                   'Topic :: Text Processing',
                   'Topic :: Utilities'],
      
      platforms = ('Any'),
      
      keywords = ('segmentation', 'similarity', 'discourse'),
      
      data_files=[('segeval/data', ['segeval/data/hearst1997.tsv',
                                    'segeval/data/hearst1997.json',
                                    'segeval/data/hearst1997_positions.csv'])],
      
      packages=['segeval',
                'segeval.data',
                'segeval.ml',
                'segeval.agreement',
                'segeval.similarity',
                'segeval.similarity.distance',
                'segeval.window'])
    



'''
Segmentation version of Arstein and Poesio's inter-coder agreement bias
[ArtsteinPoesio2008]_ that has been adapted to use Segmentation 
Similarity [FournierInkpen2012]_.

.. moduleauthor:: Chris Fournier <chris.m.fournier@gmail.com>
'''
#===============================================================================
# Copyright (c) 2011-2012, Chris Fournier
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the author nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#       
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#===============================================================================
from .Kappa import fleiss_kappa
from .Pi import fleiss_pi
from .. import compute_mean, compute_mean_values, create_tsv_rows
from ..data import load_file
from ..data.TSV import write_tsv
from ..data.Display import render_value, render_mean_values


def artstein_poesio_bias(dataset_masses):
    '''
    Artstein and Poesio's annotator bias, or B [ArtsteinPoesio2008]_
    (p. 572):
    
    .. math::
        B = A^{\pi^*}_e - A^{\kappa^*}_e
    
    :param items_masses: Segmentation masses for a collection of items where \
                        each item is multiply coded (all coders code all items).
    :type items_masses: dict
    
    :returns: B, or Bias
    :rtype: :class:`decimal.Decimal`
    
    .. seealso:: :func:`segeval.agreement.observed_agreement` for an example of\
     ``items_masses``.
    '''
    # pylint: disable=C0103
    A_pi_e     = fleiss_pi(   dataset_masses, return_parts=True)[1]
    A_fleiss_e = fleiss_kappa(dataset_masses, return_parts=True)[1]
    return A_pi_e - A_fleiss_e


def mean_artstein_poesio_bias(dataset_masses):
    '''
    Calculate mean segmentation bias.
    
    .. seealso:: :func:`artstein_poesio_bias`, :func:`segeval.compute_mean`
    
    :param dataset_masses: Segmentation mass dataset (including multiple \
                           codings).
    :type dataset_masses: dict
    
    :returns: Mean, standard deviation, and variance.
    :rtype: :class:`decimal.Decimal`, :class:`decimal.Decimal`, :class:`decimal.Decimal`
    '''
    return compute_mean(dataset_masses, artstein_poesio_bias)


OUTPUT_NAME     = 'S-based Artstein and Poesio\'s (2008) Bias'
SHORT_NAME      = 'B_s'
SHORT_NAME_MEAN = 'Mean %s' % SHORT_NAME


def values_artstein_poesio_bias(dataset_masses):
    '''
    Produces a TSV for this metric
    '''
    header = list([SHORT_NAME])
    values = compute_mean_values(dataset_masses, artstein_poesio_bias)
    return create_tsv_rows(header, values)


def parse(args):
    '''
    Parse this module's metric arguments and perform requested actions.
    '''
    output = None
    values, is_file = load_file(args)
    # Is a TSV requested?
    if args['output'] != None:
        # Create a TSV
        output_file = args['output'][0]
        header, rows = values_artstein_poesio_bias(values)
        write_tsv(output_file, header, rows)
    else:
        # Create a string to output
        if not args['output'] and is_file:
            # Render for one item
            output = render_value(SHORT_NAME, str(artstein_poesio_bias(values)))
        else:
            # Render for one or more items
            mean, std, var, stderr = mean_artstein_poesio_bias(values)
            output = render_mean_values(SHORT_NAME_MEAN, mean, std, var, stderr)
    # Return
    return output


def create_parser(subparsers):
    '''
    Setup a command line parser for this module's metric.
    '''
    from ..data import parser_add_file_support
    parser = subparsers.add_parser('b',
                                   help=OUTPUT_NAME)
    parser_add_file_support(parser)
    parser.set_defaults(func=parse)




'''
Tests the segmentation version of Arstein and Poesio's bias.

.. moduleauthor:: Chris Fournier <chris.m.fournier@gmail.com>
'''
#===============================================================================
# Copyright (c) 2011-2012, Chris Fournier
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the author nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#       
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#===============================================================================
import unittest
from decimal import Decimal
from .Bias import artstein_poesio_bias, mean_artstein_poesio_bias
from ..data.Samples import KAZANTSEVA2012_G5, KAZANTSEVA2012_G2, \
    COMPLETE_AGREEMENT


class TestBias(unittest.TestCase):
    '''
    Test Arstein and Poesio's (2008) annotator bias.
    '''
    # pylint: disable=R0904

    def test_bias_g5(self):
        '''
        Test bias upon Group 5 of Kazantseva (2012) data.
        '''
        bias = artstein_poesio_bias(KAZANTSEVA2012_G5)
        self.assertTrue(bias > 0)
        self.assertEqual(bias,
                         Decimal('0.00841453429829254475759269324'))

    def test_bias_g5_ch1(self):
        '''
        Test bias upon Group 5, Chapter 1, of Kazantseva (2012) data.
        '''
        data = KAZANTSEVA2012_G5['ch1']
        data = {'ch1' : data}
        bias = artstein_poesio_bias(data)
        self.assertTrue(bias > 0)
        self.assertEqual(bias,
                         Decimal('0.00390625000000000000000000011'))

    def test_bias_g2(self):
        '''
        Test bias upon Group 2 of Kazantseva (2012) data.
        '''
        bias = artstein_poesio_bias(KAZANTSEVA2012_G2)
        self.assertTrue(bias > 0)
        self.assertEqual(bias,
                         Decimal('0.00821695210923559041105482425'))

    def test_bias_g2_ch2(self):
        '''
        Test bias upon Group 2, Chapter 2, of Kazantseva (2012) data.
        '''
        data = KAZANTSEVA2012_G2['ch2']
        data = {'ch2' : data}
        bias = artstein_poesio_bias(data)
        self.assertTrue(bias > 0)
        self.assertEqual(bias,
                         Decimal('0.00090702947845804988662131528'))

    def test_bias_complete(self):
        '''
        Test bias upon a hypothetical dataset containing complete agreement.
        '''
        bias = artstein_poesio_bias(COMPLETE_AGREEMENT)
        self.assertTrue(bias >= 0)
        self.assertEqual(bias,
                         Decimal('0.01455229356727327645713789012'))

    def test_mean_bias(self):
        '''
        Calculate mean bias on Group 2 and 5 from the dataset
        collected in [KazantsevaSzpakowicz2012]_.
        '''
        self.assertEqual(mean_artstein_poesio_bias(
                                {'g2' : KAZANTSEVA2012_G2,
                                 'g5' : KAZANTSEVA2012_G5}),
                         (Decimal('0.008315743203764067584323758745'),
                          Decimal('0.00009879109452847717326893449501'),
                          Decimal('9.759680358134512481819312619E-9'),
                          Decimal('0.00003492792643096372046965636775')))




'''
Tests the segmentation versions of Scott's and Fleiss' Pi.

References:
    Chris Fournier and Diana Inkpen. 2012. Segmentation Similarity and
    Agreement. Submitted manuscript.
    
    Ron Artstein and Massimo Poesio. 2008. Inter-coder agreement for
    computational linguistics. Computational Linguistics, 34(4):555-596. MIT
    Press.

    Marti A. Hearst. 1997. TextTiling: Segmenting Text into Multi-paragraph
    Subtopic Passages. Computational Linguistics, 23(1):33-64.
    
    William A. Scott. 1955. Reliability of content analysis: The case of nominal
    scale coding. Public Opinion Quarterly, 19(3):321-325.
    
    Joseph L. Fleiss. 1971. Measuring nominal scale agreement among many raters.
    Psychological Bulletin, 76(5):378-382.

@author: Chris Fournier
@contact: chris.m.fournier@gmail.com
'''
#===============================================================================
# Copyright (c) 2011-2012, Chris Fournier
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the author nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#       
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#===============================================================================
import unittest
from decimal import Decimal
from .Pi import scotts_pi, fleiss_pi, mean_fleiss_pi
from ..data.Samples import KAZANTSEVA2012_G5, KAZANTSEVA2012_G2, \
    COMPLETE_AGREEMENT, LARGE_DISAGREEMENT


class TestPi(unittest.TestCase):
    '''
    Test segmentation versions of Scott's Pi and Fleiss' Multi-Pi.
    '''
    # pylint: disable=R0904
    
    SKIP = False

    def test_fliess_pi_g5(self):
        '''
        Test Pi upon Group 5 of Kazantseva (2012) data.
        '''
        if TestPi.SKIP:
            return
        # Test
        self.assertEqual(fleiss_pi(KAZANTSEVA2012_G5),
                         Decimal('0.8182766795508965420878677699'))


    def test_fliess_pi_g5_ch1(self):
        '''
        Test Pi upon Group 5, Chapter 1, of Kazantseva (2012) data.
        '''
        if TestPi.SKIP:
            return
        data = {'ch1' : KAZANTSEVA2012_G5['ch1']}
        self.assertEqual(fleiss_pi(data),
                         Decimal('0.7451990632318501170960187353'))


    def test_fleiss_pi_g2(self):
        '''
        Test Pi upon Group 2 of Kazantseva (2012) data.
        '''
        if TestPi.SKIP:
            return
        # Test
        self.assertEqual(fleiss_pi(KAZANTSEVA2012_G2),
                         Decimal('0.8856338452498481859738514723'))


    def test_fleiss_pi_g2_ch2(self):
        '''
        Test Pi upon Group 2, Chapter 2, of Kazantseva (2012) data.
        '''
        if TestPi.SKIP:
            return
        data = {'ch2' : KAZANTSEVA2012_G2['ch2']}
        # Test
        self.assertEqual(fleiss_pi(data),
                         Decimal('0.8838942307692307692307692308'))
        

    def test_fleiss_pi_large_disagree(self):
        '''
        Test Pi upon a hypothetical dataset containing large disagreement.
        '''
        if TestPi.SKIP:
            return
        data = LARGE_DISAGREEMENT
        self.assertEqual(fleiss_pi(data),
                         Decimal('-0.5757942099675148626179719687'))


    def test_scotts_pi(self):
        '''
        Test Scott's and Fleiss' Pi.
        '''
        # pylint: disable=C0324,C0103
        if TestPi.SKIP:
            return
        data1 = {'i1': {'c1' : [2,8,2,1],
                        'c2' : [2,1,7,2,1]}}
        pi1  = scotts_pi(data1)
        pi1f = fleiss_pi(data1)
        self.assertEqual(pi1,
                         Decimal('0.9030303030303030303030303031'))
        self.assertEqual(pi1,pi1f)
        data2 = {'i1': {'c1' : [2, 8, 2, 1],
                        'c2' : [11, 2]}}
        pi2  = scotts_pi(data2)
        pi2f = fleiss_pi(data2)
        self.assertEqual(pi2,
                         Decimal('0.7333333333333333333333333333'))
        self.assertEqual(pi2,pi2f)
        self.assertTrue(pi2 < pi1)
    
    
    def test_scotts_pi_complete(self):
        '''
        Test Pi upon a hypothetical dataset containing complete agreement.
        '''
        # pylint: disable=C0324,C0103
        if TestPi.SKIP:
            return
        data_complete = {'i1': {'c1' : [2,8,2,1],
                                'c2' : [2,8,2,1]}}
        pi = scotts_pi(data_complete)
        self.assertEqual(pi, 1.0)
    
    
    def test_fleiss_pi_complete(self):
        '''
        Test Pi upon a hypothetical dataset containing complete agreement.
        '''
        # pylint: disable=C0103
        if TestPi.SKIP:
            return
        data_complete = COMPLETE_AGREEMENT
        pi = fleiss_pi(data_complete)
        self.assertEqual(pi, 1.0)


    def test_mean_fleiss_pi(self):
        '''
        Calculate mean Fleiss' Pi on Group 2 and 5 from the dataset
        collected in [KazantsevaSzpakowicz2012]_.
        '''
        self.assertEqual(mean_fleiss_pi(
                                {'g2' : KAZANTSEVA2012_G2,
                                 'g5' : KAZANTSEVA2012_G5}),
                         (Decimal('0.851955262400372364030859621'),
                          Decimal('0.03367858284947582194299185120'),
                          Decimal('0.001134246942749006974258072838'),
                          Decimal('0.01190717715680865612366829138')))




'''
Tests the WindowDiff evaluation metric.

.. moduleauthor:: Chris Fournier <chris.m.fournier@gmail.com>
'''
#===============================================================================
# Copyright (c) 2011-2012, Chris Fournier
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the author nor the names of its contributors may
#       be used to endorse or promote products derived from this software
#       without specific prior written permission.
#       
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#===============================================================================
import unittest
from decimal import Decimal
from .WindowDiff import window_diff, pairwise_window_diff
from ..data.Samples import KAZANTSEVA2012_G5, KAZANTSEVA2012_G2, \
    COMPLETE_AGREEMENT, LARGE_DISAGREEMENT


class TestWindowDiff(unittest.TestCase):
    '''
    Test WindowDiff.
    '''
    # pylint: disable=R0904,C0324

    def test_identical(self):
        '''
        Test whether identical segmentations produce 0.0.
        '''
        # pylint: disable=C0324,C0103
        a = [1,1,1,1,1,2,2,2,3,3,3,3,3]
        b = [1,1,1,1,1,2,2,2,3,3,3,3,3]
        self.assertEqual(window_diff(a,b), 0.0)

    def test_no_boundaries(self):
        '''
        Test whether no segments versus some segments produce 1.0.
        '''
        # pylint: disable=C0324,C0103
        a = [1,1,1,1,1,1,1,1,1,1,1,1,1]
        b = [1,1,1,1,2,2,2,2,3,3,3,3,3]
        self.assertEqual(window_diff(a,b),
                         Decimal('0.1666666666666666666666666667'))
        self.assertEqual(window_diff(b,a),
                         Decimal('1.0'))

    def test_all_boundaries(self):
        '''
        Test whether all segments versus some segments produces 0.833
        erroneous windows.
        '''
        # pylint: disable=C0324,C0103
        a = [1,2,3,4,5,6,7,8,9,10,11,12,13]
        b = [1,1,1,1,2,2,2,2,3,3,3,3,3]
        self.assertEqual(window_diff(a,b),
                         Decimal('0.8333333333333333333333333333'))
        self.assertEqual(window_diff(b,a),
                         Decimal('0.8333333333333333333333333333'))

    def test_all_and_no_boundaries(self):
        '''
        Test whether all segments versus no segments produces 1.0.
        '''
        # pylint: disable=C0324,C0103
        a = [1,2,3,4,5,6,7,8,9,10,11,12,13]
        b = [1,1,1,1,1,1,1,1,1,1,1,1,1]
        self.assertEqual(window_diff(a,b), 1.0)
        self.assertEqual(window_diff(b,a), 1.0)

    def test_translated_boundary(self):
        '''
        Test whether 2/3 total segments participate in mis-alignment produces
        0.167.
        '''
        # pylint: disable=C0324,C0103
        a = [1,1,1,1,1,2,2,2,3,3,3,3,3]
        b = [1,1,1,1,2,2,2,2,3,3,3,3,3]
        self.assertEqual(window_diff(a,b),
                         Decimal('0.1666666666666666666666666667'))
        self.assertEqual(window_diff(b,a),
                         Decimal('0.1666666666666666666666666667'))
    
    def test_extra_boundary(self):
        '''
        Test whether 1/3 segments that are non-existent produces 0.083.
        '''
        # pylint: disable=C0324,C0103
        a = [1,1,1,1,1,2,2,2,3,3,3,3,3]
        b = [1,1,1,1,1,2,3,3,4,4,4,4,4]
        self.assertEqual(window_diff(a,b),
                         Decimal('0.08333333333333333333333333333'))
        self.assertEqual(window_diff(b,a),
                         Decimal('0.08333333333333333333333333333'))
    
    def test_full_miss_and_misaligned(self):
        '''
        Test whether a full miss and a translated boundary out of 4 produces
        0.25. 
        '''
        # pylint: disable=C0324,C0103
        a = [1,1,1,1,2,2,2,2,3,3,3,3,3]
        b = [1,1,1,1,1,2,3,3,4,4,4,4,4]
        self.assertEqual(window_diff(a,b),
                         Decimal('0.25'))
        self.assertEqual(window_diff(b,a),
                         Decimal('0.25'))
    
    def test_fn_vs_fp(self):
        '''
        Test the difference between FP and FN.
        '''
        # pylint: disable=C0324,C0103
        a = [1,1,1,1,1,2,2,2,3,3,3,3,3]
        b = [1,1,1,1,1,2,3,3,4,4,4,4,4]
        self.assertEqual(window_diff(a,b),
                         Decimal('0.08333333333333333333333333333'))
        self.assertEqual(window_diff(b,a),
                         Decimal('0.08333333333333333333333333333'))
        a = [1,1,1,1,1,2,3,3,4,4,4,4,4]
        b = [1,1,1,1,1,2,2,2,3,3,3,3,3]
        self.assertEqual(window_diff(a,b),
                         Decimal('0.08333333333333333333333333333'))
        self.assertEqual(window_diff(b,a),
                         Decimal('0.08333333333333333333333333333'))


class TestPairwiseWindowDiff(unittest.TestCase):
    # pylint: disable=R0904
    '''
    Test pairwise WindowDiff.
    '''
        
    
    def test_kazantseva2012_g5(self):
        '''
        Calculate permuted pairwise WindowDiff on Group 5 from the dataset
        collected in [KazantsevaSzpakowicz2012]_.
        '''
        self.assertEqual(pairwise_window_diff(KAZANTSEVA2012_G5,
                                              lamprier_et_al_2007_fix=False),
                         (Decimal('0.3604506237259865251208907685'),
                          Decimal('0.1674103189012695088955568781'),
                          Decimal('0.02802621487462475498810473917'),
                          Decimal('0.02416359817069226127768032185')))
        self.assertEqual(pairwise_window_diff(KAZANTSEVA2012_G5,
                                              lamprier_et_al_2007_fix=True),
                         (Decimal('0.3389327650117214732278073273'),
                          Decimal('0.1644218164415795205498766705'),
                          Decimal('0.02703453372194846954943255446'),
                          Decimal('0.02373224499579829290476994223')))
    
    def test_kazantseva2012_g2(self):
        '''
        Calculate mean permuted pairwise WindowDiff on Group 2 from the dataset
        collected in [KazantsevaSzpakowicz2012]_.
        '''
        self.assertEqual(pairwise_window_diff(KAZANTSEVA2012_G2,
                                              lamprier_et_al_2007_fix=False),
                         (Decimal('0.2545512423455600226641754513'),
                          Decimal('0.1227764444488944596833784806'),
                          Decimal('0.01507405531151246718395938745'),
                          Decimal('0.01120790469248990475787117304')))
        self.assertEqual(pairwise_window_diff(KAZANTSEVA2012_G2,
                                              lamprier_et_al_2007_fix=True),
                         (Decimal('0.2282101219734164495559781364'),
                          Decimal('0.09824964933538713480770055124'),
                          Decimal('0.009652993594526537660429053508'),
                          Decimal('0.008968924867993997594174187059')))
    
    def test_large_disagreement(self):
        '''
        Calculate mean permuted pairwise WindowDiff on a theoretical dataset
        containing large disagreement.
        '''
        self.assertEqual(pairwise_window_diff(LARGE_DISAGREEMENT,
                                              lamprier_et_al_2007_fix=False),
                         (1.0,
                          0.0,
                          0.0,
                          0.0))
        self.assertEqual(pairwise_window_diff(LARGE_DISAGREEMENT,
                                              lamprier_et_al_2007_fix=True),
                          (Decimal('0.8583107329225146711972779545'),
                           Decimal('0.1040894435363239488284336823'),
                           Decimal('0.01083461225570157148986770720'),
                           Decimal('0.03680117568723445601862258284')))
    
    def test_complete_agreement(self):
        '''
        Calculate mean permuted pairwise WindowDiff on a theoretical dataset
        containing complete agreement.
        '''
        self.assertEqual(pairwise_window_diff(COMPLETE_AGREEMENT,
                                              lamprier_et_al_2007_fix=False),
                         (0.0,
                          0.0,
                          0.0,
                          0.0))
        self.assertEqual(pairwise_window_diff(LARGE_DISAGREEMENT,
                                              lamprier_et_al_2007_fix=True),
                          (Decimal('0.8583107329225146711972779545'),
                           Decimal('0.1040894435363239488284336823'),
                           Decimal('0.01083461225570157148986770720'),
                           Decimal('0.03680117568723445601862258284')))

